@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: threads.c,v 1.7 2004/10/09 11:36:36 shirok Exp $
+ *  $Id: threads.c,v 1.8 2005/05/12 08:52:53 shirok Exp $
  */
 
 #include <gauche.h>
@@ -69,7 +69,7 @@ ScmObj Scm_MakeThread(ScmProcedure *thunk, ScmObj name)
     if (SCM_PROCEDURE_REQUIRED(thunk) != 0) {
         Scm_Error("thunk required, but got %S", thunk);
     }
-    vm = Scm_NewVM(current, current->module, name);
+    vm = Scm_NewVM(current, name);
     vm->thunk = thunk;
     vm->defaultEscapeHandler = SCM_OBJ(&thread_error_handler_STUB);
     return SCM_OBJ(vm);
