@@ -1,7 +1,7 @@
 ;;;
 ;;; procedure.scm - auxiliary procedure utilities.  to be autoloaded.
 ;;;  
-;;;   Copyright (c) 2000-2003 Shiro Kawai, All rights reserved.
+;;;   Copyright (c) 2000-2005 Shiro Kawai, All rights reserved.
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: procedure.scm,v 1.11 2004/11/12 07:07:42 shirok Exp $
+;;;  $Id: procedure.scm,v 1.13 2005/05/26 09:57:49 shirok Exp $
 ;;;
 
 (define-module gauche.procedure
@@ -41,7 +41,7 @@
           let-optionals* let-keywords* get-optional
           arity procedure-arity-includes?
           <arity-at-least> arity-at-least? arity-at-least-value
-          case-lambda
+          case-lambda disasm
           ))
 
 (select-module gauche.procedure)
@@ -51,11 +51,14 @@
 (define (pa$ fn . args)                  ;partial apply
   (lambda more-args (apply fn (append args more-args))))
 
-(define (compose f g . more)
-  (if (null? more)
-      (lambda args
-        (call-with-values (lambda () (apply g args)) f))
-      (compose f (apply compose g more))))
+(define (compose . fns)
+  (cond ((null? fns) values)
+        ((null? (cdr fns)) (car fns))
+        ((null? (cddr fns))
+         (lambda args
+           (call-with-values (lambda () (apply (cadr fns) args)) (car fns))))
+        (else
+         (compose (car fns) (apply compose (cdr fns))))))
 
 (define (compose$ f) (pa$ compose f))
 
@@ -206,4 +209,15 @@
             (else
              (error "wrong number of arguments to case-lambda:" args))))))
 
+;; disassembler.
+;; I'm not sure whether this should be here or not, but fot the time being...
+
+(define (disasm proc)
+  ;; kludge
+  (let ((dumper (if (find-module 'gauche.internal)
+                  (eval '(with-module gauche.internal vm-dump-code)
+                        (find-module 'gauche))
+                  vm-dump-code)))
+    (dumper (closure-code proc))))
+
 (provide "gauche/procedure")
