@@ -1,7 +1,7 @@
 ;;;
 ;;; SRFI-0   feature based conditional expansion construct
 ;;;
-;;; $Id: srfi-0.scm,v 1.22 2004/07/28 05:13:22 shirok Exp $
+;;; $Id: srfi-0.scm,v 1.25 2005/05/24 09:52:56 shirok Exp $
 ;;;
 
 (define-module srfi-0
@@ -60,9 +60,12 @@
                 (srfi-29 srfi-29)
                 (srfi-30)
                 (srfi-31)
+                (srfi-34)
+                (srfi-35)
                 (srfi-37 srfi-37)
                 (srfi-38)
                 (srfi-39 gauche.parameter)
+                (srfi-42 srfi-42)
                 )))
 
 (define-macro (cond-expand . clauses)
@@ -73,7 +76,7 @@
   ;; by Gauche built-in features).
   (define (fulfill? req seed)
     (cond
-     ((identifier? req) (fulfill? (identifier->symbol req)))
+     ((identifier? req) (fulfill? (identifier->symbol req) seed))
      ((symbol? req)
       (let ((p (assq req *cond-features*)))
         (and p (if (null? (cdr p)) seed (cons (cadr p) seed)))))
