@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche-init.scm - initialize standard environment
 ;;;
-;;;   Copyright (c) 2000-2003 Shiro Kawai, All rights reserved.
+;;;   Copyright (c) 2000-2005 Shiro Kawai, All rights reserved.
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: gauche-init.scm,v 1.117 2004/10/09 11:36:37 shirok Exp $
+;;;  $Id: gauche-init.scm,v 1.120 2005/05/22 03:27:32 shirok Exp $
 ;;;
 
 (select-module gauche)
@@ -47,16 +47,31 @@
 (define-macro (add-load-path path . args)
   `',(apply %add-load-path path args))
 
-;; Same as above.
-(define-macro (require feature)
-  `',(%require feature))
-
-(define-macro (export-all)
-  `',(%export-all))
-
 (define-macro (autoload file . vars)
   `(%autoload (current-module) ',file ',vars))
 
+;; Transition to the new compiler.
+;; The new compiler supports these syntax built-in.  We need a macro
+;; version when we compile the new one with the old gosh.
+(define-macro (ensure-extend)
+  (if (symbol-bound? '%extend-module)
+    #f
+    '(begin
+       (define-macro (require feature)
+         `',(%require feature))
+       (define-macro (export-all)
+         `',(%export-all))
+       (define-macro (extend . modules)
+         (%extend (map (lambda (m)
+                         (or (find-module m)
+                             (begin
+                               (%require (module-name->path m))
+                               (find-module m))
+                             (error "undefined module" m)))
+                       modules))
+         #f))))
+(ensure-extend)
+
 ;; Preferred way
 ;;  (use x.y.z) ==> (require "x/y/z") (import x.y.z)
 
@@ -67,69 +82,31 @@
      (import ,module)))
 
 ;; create built-in modules, so that (use srfi-6) won't complain, for example.
+(define-module srfi-2 )
 (define-module srfi-6 )
 (define-module srfi-8 )
 (define-module srfi-10 )
 (define-module srfi-17 )
 
+;; for backward compatibility
+(define-module gauche.vm.debugger )
+
 ;;
 ;; Auxiliary definitions
 ;;
 
-(define-in-module scheme call/cc call-with-current-continuation)
-
-(define-in-module scheme (call-with-values producer consumer)
-  (receive vals (producer) (apply consumer vals)))
-
 (define <exception> <condition>) ;; backward compatibility
 
 (define-reader-ctor 'string-interpolate
   (lambda (s) (string-interpolate s))) ;;lambda is required to delay loading
 
-;; srfi-17
-(define (getter-with-setter get set)
-  (let ((proc (lambda x (apply get x))))
-    (set! (setter proc) set)
-    proc))
-
-;; print (as in SCM, Chicken)
-(define (print . args) (for-each display args) (newline))
-
-;; srfi-38
-(define read-with-shared-structure read)
-(define read/ss read)
-
-(define (write-with-shared-structure obj . args)
-  (write* obj (if (pair? args) (car args) (current-output-port))))
-(define write/ss write-with-shared-structure)
-
-;; format
-(define format (undefined))
-(define format/ss (undefined))
-(let ((format-int
-       (lambda (port fmt args shared?)
-         (cond ((eqv? port #f)
-                (let ((out (open-output-string :private? #t)))
-                  (%format out fmt args shared?)
-                  (get-output-string out)))
-               ((eqv? port #t)
-                (%format (current-output-port) fmt args shared?))
-               (else (%format port fmt args shared?))))))
-  (set! format
-        (lambda (fmt . args)
-          (if (string? fmt)
-            (format-int #f fmt args #f) ;; srfi-28 compatible behavior
-            (format-int fmt (car args) (cdr args) #f))))
-  (set! format/ss
-        (lambda (fmt . args)
-          (if (string? fmt)
-            (format-int #f fmt args #t) ;; srfi-28 compatible behavior
-            (format-int fmt (car args) (cdr args) #t))))
-  )
-
 ;;
 ;; Load object system
 ;;
 
-(require "gauche/object")
+;; A trick to cross-compile during development.  Will go away.
+(define-macro (%init-object-system)
+  (unless (find-module 'gauche.object)
+    (%require "gauche/object")))
 
+(%init-object-system)
