@@ -1,7 +1,7 @@
 /*
  * load.c - load a program
  *
- *   Copyright (c) 2000-2004 Shiro Kawai, All rights reserved.
+ *   Copyright (c) 2000-2005 Shiro Kawai, All rights reserved.
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: load.c,v 1.94 2004/11/21 12:46:59 shirok Exp $
+ *  $Id: load.c,v 1.98 2005/05/17 04:33:09 shirok Exp $
  */
 
 #include <stdlib.h>
@@ -89,7 +89,8 @@ static ScmObj key_ignore_coding      = SCM_UNBOUND;
  * 
  *   The most basic function in the load()-family.  Read an expression
  *   from the given port and evaluates it repeatedly, until it reaches
- *   EOF.  Then the port is closed.
+ *   EOF.  Then the port is closed.   The port is locked by the calling
+ *   thread until the operation terminates.
  *
  *   The result of the last evaluation remains on VM.
  *
@@ -100,6 +101,9 @@ static ScmObj key_ignore_coding      = SCM_UNBOUND;
  *   FLAGS argument is ignored for now, but reserved for future
  *   extension.  SCM_LOAD_QUIET_NOFILE and SCM_LOAD_IGNORE_CODING
  *   won't have any effect for LoadFromPort; see Scm_Load below.
+ *
+ *   TODO: if we're using coding-aware port, how should we propagate
+ *   locking into the wrapped (original) port?
  */
 
 struct load_packet {
@@ -109,6 +113,7 @@ struct load_packet {
     ScmObj prev_port;
     ScmObj prev_history;
     ScmObj prev_next;
+    int    prev_situation;
 };
 
 /* Clean up */
@@ -117,10 +122,12 @@ static ScmObj load_after(ScmObj *args, int nargs, void *data)
     struct load_packet *p = (struct load_packet *)data;
     ScmVM *vm = Scm_VM();
     Scm_ClosePort(p->port);
+    PORT_UNLOCK(p->port);
     Scm_SelectModule(p->prev_module);
     vm->load_port = p->prev_port;
     vm->load_history = p->prev_history;
     vm->load_next = p->prev_next;
+    vm->evalSituation = p->prev_situation;
     return SCM_UNDEFINED;
 }
 
@@ -132,7 +139,7 @@ static ScmObj load_cc(ScmObj result, void **data)
 
     if (!SCM_EOFP(expr)) {
         Scm_VMPushCC(load_cc, data, 1);
-        return Scm_VMEval(expr, SCM_UNBOUND);
+        return Scm_VMEval(expr, SCM_FALSE);
     } else {
         return SCM_TRUE;
     }
@@ -168,6 +175,7 @@ ScmObj Scm_VMLoadFromPort(ScmPort *port, ScmObj next_paths,
     p->prev_port = vm->load_port;
     p->prev_history = vm->load_history;
     p->prev_next = vm->load_next;
+    p->prev_situation = vm->evalSituation;
 
     SCM_READ_CONTEXT_INIT(&(p->ctx));
     p->ctx.flags = SCM_READ_LITERAL_IMMUTABLE | SCM_READ_SOURCE_INFO;
@@ -178,6 +186,7 @@ ScmObj Scm_VMLoadFromPort(ScmPort *port, ScmObj next_paths,
     vm->load_next = next_paths;
     vm->load_port = SCM_OBJ(port);
     vm->module = module;
+    vm->evalSituation = SCM_VM_LOADING;
     if (SCM_PORTP(p->prev_port)) {
         port_info = SCM_LIST2(p->prev_port,
                               Scm_MakeInteger(Scm_PortLine(SCM_PORT(p->prev_port))));
@@ -185,6 +194,8 @@ ScmObj Scm_VMLoadFromPort(ScmPort *port, ScmObj next_paths,
         port_info = SCM_LIST1(SCM_FALSE);
     }
     vm->load_history = Scm_Cons(port_info, vm->load_history);
+
+    PORT_LOCK(port, vm);
     return Scm_VMDynamicWindC(NULL, load_body, load_after, p);
 }
 
@@ -386,7 +397,7 @@ int Scm_Load(const char *cpath, int flags)
     ScmObj r, f = SCM_MAKE_STR_COPYING(cpath);
     ScmObj options = SCM_NIL;
     
-    if (!(flags&SCM_LOAD_QUIET_NOFILE)) {
+    if (flags&SCM_LOAD_QUIET_NOFILE) {
         options = Scm_Cons(key_error_if_not_found,
                            Scm_Cons(SCM_FALSE, options));
     }
@@ -786,7 +797,7 @@ ScmObj Scm_DynLoad(ScmString *filename, ScmObj initfn, int export_)
  *   There may be a case that the feature dependency forms a loop because
  *   of bug.  An error should be signaled in such a case, rather than going
  *   to deadlock.   So, when the calling thread finds the required feature
- *   is in the ldinfo.providing alist, it checks the waiting chaing of
+ *   is in the ldinfo.providing alist, it checks the waiting chain of
  *   features, and no threads are waiting for a feature being provided by
  *   the calling thread.
  */
@@ -797,7 +808,7 @@ ScmObj Scm_Require(ScmObj feature)
     ScmVM *vm = Scm_VM();
     ScmObj provided, providing, p, q;
     int loop = FALSE;
-    
+
     if (!SCM_STRINGP(feature)) {
         Scm_Error("require: string expected, but got %S\n", feature);
     }
@@ -1098,11 +1109,13 @@ void Scm__InitLoad(void)
     DEF(ldinfo.load_suffixes_rec, SCM_SYM_LOAD_SUFFIXES, init_load_suffixes);
     DEF(ldinfo.cond_features_rec, SCM_SYM_COND_FEATURES, init_cond_features);
 
-    ldinfo.provided = SCM_LIST4(SCM_MAKE_STR("srfi-6"), /* string ports (builtin) */
-                                SCM_MAKE_STR("srfi-8"), /* receive (builtin) */
-                                SCM_MAKE_STR("srfi-10"), /* #, (builtin) */
-                                SCM_MAKE_STR("srfi-17")  /* set! (builtin) */
-        );
+    ldinfo.provided =
+        SCM_LIST5(SCM_MAKE_STR("srfi-2"), /* and-let* */
+                  SCM_MAKE_STR("srfi-6"), /* string ports (builtin) */
+                  SCM_MAKE_STR("srfi-8"), /* receive (builtin) */
+                  SCM_MAKE_STR("srfi-10"), /* #, (builtin) */
+                  SCM_MAKE_STR("srfi-17")  /* set! (builtin) */
+            );
     ldinfo.providing = SCM_NIL;
     ldinfo.waiting = SCM_NIL;
     ldinfo.dso_suffixes = SCM_LIST2(SCM_MAKE_STR(".la"),
