@@ -1,7 +1,7 @@
 /*
  * signal.c - signal handling
  *
- *   Copyright (c) 2000-2003 Shiro Kawai, All rights reserved.
+ *   Copyright (c) 2000-2004 Shiro Kawai, All rights reserved.
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: signal.c,v 1.28 2004/02/02 10:43:37 shirok Exp $
+ *  $Id: signal.c,v 1.31 2004/11/22 14:12:43 shirok Exp $
  */
 
 #include <stdlib.h>
@@ -64,11 +64,17 @@
  *  since GC uses it in the Linux/pthread environment.
  */
 
-#ifdef GAUCHE_USE_PTHREADS
-#define SIGPROCMASK pthread_sigmask
-#else
-#define SIGPROCMASK sigprocmask
-#endif
+#ifndef __MINGW32__
+  #ifdef GAUCHE_USE_PTHREADS
+  #define SIGPROCMASK pthread_sigmask
+  #else
+  #define SIGPROCMASK sigprocmask
+  #endif
+#else  /* __MINGW32__ */
+  /* This isn't correct (we need some mechanism to block the signal),
+     but just for the time being ... */
+  #define SIGPROCMASK(mode, set, omask)  (0)
+#endif /* __MINGW32__ */
 
 /* Master signal handler vector. */
 static struct sigHandlersRec {
@@ -98,9 +104,13 @@ static struct sigdesc {
     int num;
     int defaultHandle;
 } sigDesc[] = {
+#ifdef SIGHUP
     SIGDEF(SIGHUP,  SIGDEF_EXIT),     /* Hangup (POSIX) */
+#endif
     SIGDEF(SIGINT,  SIGDEF_ERROR),    /* Interrupt (ANSI) */
+#ifdef SIGQUIT
     SIGDEF(SIGQUIT, SIGDEF_EXIT),     /* Quit (POSIX) */
+#endif
     SIGDEF(SIGILL,  SIGDEF_NOHANDLE), /* Illegal instruction (ANSI) */
 #ifdef SIGTRAP
     SIGDEF(SIGTRAP, SIGDEF_ERROR),    /* Trace trap */
@@ -113,22 +123,44 @@ static struct sigdesc {
     SIGDEF(SIGBUS,  SIGDEF_NOHANDLE), /* BUS error (4.2 BSD) */
 #endif
     SIGDEF(SIGFPE,  SIGDEF_ERROR),    /* Floating-point exception (ANSI) */
+#ifdef SIGKILL
     SIGDEF(SIGKILL, SIGDEF_NOHANDLE), /* Kill, unblockable (POSIX) */
+#endif
+#ifdef SIGUSR1
     SIGDEF(SIGUSR1, SIGDEF_ERROR),    /* User-defined signal 1 (POSIX) */
+#endif
     SIGDEF(SIGSEGV, SIGDEF_NOHANDLE), /* Segmentation violation (ANSI) */
+#ifdef SIGUSR2
     SIGDEF(SIGUSR2, SIGDEF_ERROR),    /* User-defined signal 2 (POSIX) */
+#endif
+#ifdef SIGPIPE
     SIGDEF(SIGPIPE, SIGDEF_ERROR),    /* Broken pipe (POSIX) */
+#endif
+#ifdef SIGALRM
     SIGDEF(SIGALRM, SIGDEF_ERROR),    /* Alarm clock (POSIX) */
+#endif
     SIGDEF(SIGTERM, SIGDEF_EXIT),     /* Termination (ANSI) */
 #ifdef SIGSTKFLT
     SIGDEF(SIGSTKFLT, SIGDEF_ERROR),  /* Stack fault */
 #endif
+#ifdef SIGCHLD
     SIGDEF(SIGCHLD, SIGDEF_NOHANDLE), /* Child status has changed (POSIX) */
+#endif
+#ifdef SIGCONT
     SIGDEF(SIGCONT, SIGDEF_NOHANDLE), /* Continue (POSIX) */
+#endif
+#ifdef SIGSTOP
     SIGDEF(SIGSTOP, SIGDEF_NOHANDLE), /* Stop, unblockable (POSIX) */
+#endif
+#ifdef SIGTSTP
     SIGDEF(SIGTSTP, SIGDEF_NOHANDLE), /* Keyboard stop (POSIX) */
+#endif
+#ifdef SIGTTIN
     SIGDEF(SIGTTIN, SIGDEF_NOHANDLE), /* Background read from tty (POSIX) */
+#endif
+#ifdef SIGTTOU
     SIGDEF(SIGTTOU, SIGDEF_NOHANDLE), /* Background write to tty (POSIX) */
+#endif
 #ifdef SIGURG
     SIGDEF(SIGURG,  SIGDEF_NOHANDLE), /* Urgent condition on socket (4.2 BSD) */
 #endif
@@ -329,6 +361,10 @@ ScmObj Scm_SysSigsetFill(ScmSysSigset *set, int emptyp)
     return SCM_OBJ(set);
 }
 
+/*
+ * Signal handling
+ */
+
 /*
  * System's signal handler - just enqueue the signal
  */
@@ -340,19 +376,29 @@ static void sig_handle(int signum)
        terminating and in the cleanup phase. */
     if (vm == NULL) return;
     
-    if (vm->sigOverflow) return;
-    
-    if (vm->sigQueueHead <= vm->sigQueueTail) {
-        vm->sigQueue[vm->sigQueueTail++] = signum;
-        if (vm->sigQueueTail >= SCM_VM_SIGQ_SIZE) {
-            vm->sigQueueTail = 0;
-        }
-    } else {
-        vm->sigQueue[vm->sigQueueTail++] = signum;
+    if (vm->sigq.overflow) {
+        /* TODO: what should we do? */
+        Scm_Warn("Signal queue overflow\n");
+        return;
     }
-    if (vm->sigQueueTail == vm->sigQueueHead) {
-        Scm_Error("signal queue overflow\n");
+    vm->sigq.queue[vm->sigq.tail++] = signum;
+    if (vm->sigq.tail >= SCM_VM_SIGQ_SIZE) {
+        vm->sigq.tail = 0;
     }
+    if (vm->sigq.tail == vm->sigq.head) vm->sigq.overflow++;
+    vm->queueNotEmpty |= SCM_VM_SIGQ_MASK;
+}
+
+/*
+ * Initializes signal queue
+ */
+void Scm_SignalQueueInit(ScmSignalQueue* q)
+{
+    int i;
+    for (i=0; i<SCM_VM_SIGQ_SIZE; i++) q->queue[i] = -1;
+    q->head = q->tail = 0;
+    q->overflow = 0;
+    q->pending = SCM_NIL;
 }
 
 /*
@@ -362,6 +408,7 @@ static void sig_handle(int signum)
 void Scm_SigCheck(ScmVM *vm)
 {
     ScmObj tail, cell, sp;
+    ScmSignalQueue *q = &vm->sigq;
     sigset_t omask;
     int sigQsize, i;
     int sigQcopy[SCM_VM_SIGQ_SIZE]; /* copy of signal queue */
@@ -369,17 +416,18 @@ void Scm_SigCheck(ScmVM *vm)
     /* Copy VM's signal queue to local storage, for we can't call
        storage allocation during blocking signals. */
     SIGPROCMASK(SIG_BLOCK, &sigHandlers.masterSigset, &omask);
-    for (sigQsize = 0; vm->sigQueueHead != vm->sigQueueTail; sigQsize++) {
-        sigQcopy[sigQsize] = vm->sigQueue[vm->sigQueueHead++];
-        if (vm->sigQueueHead >= SCM_VM_SIGQ_SIZE) vm->sigQueueHead = 0;
+    for (sigQsize = 0; q->head != q->tail; sigQsize++) {
+        sigQcopy[sigQsize] = q->queue[q->head++];
+        if (q->head >= SCM_VM_SIGQ_SIZE) q->head = 0;
     }
-    vm->sigOverflow = 0; /*TODO: we should do something*/
+    q->overflow = 0; /*TODO: we should do something*/
+    vm->queueNotEmpty &= ~SCM_VM_SIGQ_MASK;
     SIGPROCMASK(SIG_SETMASK, &omask, NULL);
 
     /* Now, prepare queued signal handlers
        If an error is thrown in this loop, the queued signals will be
        lost---it doesn't look like so, but I may overlook something. */
-    tail = vm->sigPending;
+    tail = q->pending;
     if (!SCM_NULLP(tail)) tail = Scm_LastPair(tail);
     for (i=0; i<sigQsize; i++) {
         if (SCM_PROCEDUREP(sigHandlers.handlers[sigQcopy[i]])) {
@@ -387,7 +435,7 @@ void Scm_SigCheck(ScmVM *vm)
                              SCM_MAKE_INT(sigQcopy[i]),
                              SCM_NIL);
             if (SCM_NULLP(tail)) {
-                vm->sigPending = tail = cell;
+                q->pending = tail = cell;
             } else {
                 SCM_SET_CDR(tail, cell);
                 tail = SCM_CDR(tail);
@@ -399,9 +447,9 @@ void Scm_SigCheck(ScmVM *vm)
        of those handlers, the rest of handlers remain in the queue. */
     /* TODO: if VM is active, it'd be better to make the active VM to handle
        those handler procs, instead of calling Scm_Eval. */
-    SCM_FOR_EACH(sp, vm->sigPending) {
+    SCM_FOR_EACH(sp, q->pending) {
         ScmObj h = SCM_CAR(sp);
-        vm->sigPending = SCM_CDR(sp);
+        q->pending = SCM_CDR(sp);
         Scm_Apply(SCM_CAR(h), SCM_LIST1(SCM_CDR(h)));
     }
 }
@@ -572,11 +620,12 @@ ScmObj Scm_SysSigmask(int how, ScmSysSigset *newmask)
  */
 static void scm_sigsuspend(sigset_t *mask)
 {
+#ifndef __MINGW32__
     sigset_t omask;
     ScmVM *vm = Scm_VM();
     for (;;) {
         SIGPROCMASK(SIG_BLOCK, &sigHandlers.masterSigset, &omask);
-        if (vm->sigQueueTail != vm->sigQueueHead) {
+        if (vm->sigq.tail != vm->sigq.head) {
             SIGPROCMASK(SIG_SETMASK, &omask, NULL);
             Scm_SigCheck(vm);
             continue;
@@ -586,6 +635,9 @@ static void scm_sigsuspend(sigset_t *mask)
     sigsuspend(mask);
     SIGPROCMASK(SIG_SETMASK, &omask, NULL);
     SCM_SIGCHECK(vm);
+#else  /*__MINGW32__*/
+    Scm_Error("sigsuspend not supported on MinGW port");
+#endif /*__MINGW32__*/
 }
 
 ScmObj Scm_SigSuspend(ScmSysSigset *mask)
@@ -609,6 +661,26 @@ ScmObj Scm_Pause(void)
     return SCM_UNDEFINED;
 }
 
+/*
+ * Emulation stubs for Windows/MinGW
+ */
+#ifdef __MINGW32__
+
+int sigaction(int signum, const struct sigaction *act,
+	      struct sigaction *oact)
+{
+    if (oact != NULL) {
+	Scm_Panic("sigaction() with oldact != NULL isn't supported on MinGW port");
+    }
+    if (signal(signum, act->sa_handler) == SIG_ERR) {
+	return -1;
+    } else {
+	return 0;
+    }
+}
+#endif /* __MINGW32__ */
+
+
 /*
  * initialize
  */
@@ -624,8 +696,8 @@ void Scm__InitSignal(void)
     sigemptyset(&sigHandlers.masterSigset);
     for (i=0; i<NSIG; i++) sigHandlers.handlers[i] = SCM_FALSE;
     
-    Scm_InitBuiltinClass(&Scm_SysSigsetClass, "<sys-sigset>", NULL,
-                         sizeof(ScmSysSigset), mod);
+    Scm_InitStaticClass(&Scm_SysSigsetClass, "<sys-sigset>",
+                        mod, NULL, 0);
 
     for (desc = sigDesc; desc->name; desc++) {
         SCM_DEFINE(mod, desc->name, SCM_MAKE_INT(desc->num));
