@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: system.c,v 1.61 2004/10/09 11:36:37 shirok Exp $
+ *  $Id: system.c,v 1.63 2005/05/24 23:28:38 shirok Exp $
  */
 
 #include <stdio.h>
@@ -43,6 +43,7 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <fcntl.h>
 #ifndef __MINGW32__
 #include <grp.h>
 #include <pwd.h>
@@ -50,7 +51,6 @@
 #include <windows.h>
 #include <lm.h>
 #include <tlhelp32.h>
-#include <fcntl.h>
 #endif  /*__MINGW32__*/
 
 #define LIBGAUCHE_BODY
@@ -376,27 +376,19 @@ ScmObj Scm_DirName(ScmString *filename)
 }
 
 /* Make mkstemp() work even if the system doesn't have one. */
-ScmObj Scm_SysMkstemp(ScmString *template)
+int Scm_Mkstemp(char *templat)
 {
-#define MKSTEMP_PATH_MAX 1025  /* Geez, remove me */
-    char name[MKSTEMP_PATH_MAX];
-    ScmObj sname;
-    int siz = SCM_STRING_SIZE(template), fd;
-    if (siz >= MKSTEMP_PATH_MAX-6) { 
-	Scm_Error("pathname too long: %S", template);
-    }
-    memcpy(name, SCM_STRING_START(template), siz);
+    int fd = -1;
 #if defined(HAVE_MKSTEMP)
-    memcpy(name + siz, "XXXXXX", 6);
-    name[siz+6] = '\0';
-    SCM_SYSCALL(fd, mkstemp(name));
+    SCM_SYSCALL(fd, mkstemp(templat));
     if (fd < 0) Scm_SysError("mkstemp failed");
-    sname = SCM_MAKE_STR_COPYING(name);
-    SCM_RETURN(Scm_Values2(Scm_MakePortWithFd(sname, SCM_PORT_OUTPUT, fd,
-					      SCM_PORT_BUFFER_FULL, TRUE),
-			   sname));
+    return fd;
 #else   /*!defined(HAVE_MKSTEMP)*/
     /* Emulate mkstemp. */
+    int siz = strlen(templat);
+    if (siz < 6) {
+        Scm_Error("mkstemp - invalid template: %s", templat);
+    }
 #define MKSTEMP_MAX_TRIALS 65535   /* avoid infinite loop */
     {
 	u_long seed = (u_long)time(NULL);
@@ -404,11 +396,11 @@ ScmObj Scm_SysMkstemp(ScmString *template)
 	char suffix[7];
 	for (numtry=0; numtry<MKSTEMP_MAX_TRIALS; numtry++) {
 	    snprintf(suffix, 7, "%06x", seed&0xffffff);
-	    memcpy(name+siz, suffix, 7);
+	    memcpy(templat+siz-6, suffix, 7);
 #ifndef __MINGW32__
-	    SCM_SYSCALL(fd, open(name, O_CREAT|O_EXCL|O_WRONLY, 0600));
+	    SCM_SYSCALL(fd, open(templat, O_CREAT|O_EXCL|O_WRONLY, 0600));
 #else  /*__MINGW32__*/
- 	    SCM_SYSCALL(fd, open(name, _O_CREAT|_O_EXCL|_O_WRONLY, 0600));
+ 	    SCM_SYSCALL(fd, open(templat, _O_CREAT|_O_EXCL|_O_WRONLY, 0600));
 #endif /*__MINGW32__*/
 	    if (fd >= 0) break;
 	    seed *= 2654435761UL;
@@ -416,14 +408,31 @@ ScmObj Scm_SysMkstemp(ScmString *template)
 	if (numtry == MKSTEMP_MAX_TRIALS) {
 	    Scm_Error("mkstemp failed");
 	}
-	sname = SCM_MAKE_STR_COPYING(name);
-	SCM_RETURN(Scm_Values2(Scm_MakePortWithFd(sname, SCM_PORT_OUTPUT, fd,
-						  SCM_PORT_BUFFER_FULL, TRUE),
-			       sname));
     }
+    return fd;
 #endif /*!defined(HAVE_MKSTEMP)*/
 }
 
+
+ScmObj Scm_SysMkstemp(ScmString *templat)
+{
+#define MKSTEMP_PATH_MAX 1025  /* Geez, remove me */
+    char name[MKSTEMP_PATH_MAX];
+    ScmObj sname;
+    int siz = SCM_STRING_SIZE(templat), fd;
+    if (siz >= MKSTEMP_PATH_MAX-6) { 
+	Scm_Error("pathname too long: %S", templat);
+    }
+    memcpy(name, SCM_STRING_START(templat), siz);
+    memcpy(name + siz, "XXXXXX", 6);
+    name[siz+6] = '\0';
+    fd = Scm_Mkstemp(name);
+    sname = SCM_MAKE_STR_COPYING(name);
+    SCM_RETURN(Scm_Values2(Scm_MakePortWithFd(sname, SCM_PORT_OUTPUT, fd,
+					      SCM_PORT_BUFFER_FULL, TRUE),
+			   sname));
+}
+
 /*===============================================================
  * Stat (sys/stat.h)
  */
@@ -969,7 +978,7 @@ ScmObj Scm_SysExec(ScmString *file, ScmObj args, ScmObj iomap, int forkp)
     char **argv;
     const char *program;
     ScmObj ap, iop;
-    pid_t pid;
+    pid_t pid = 0;
 
     if (argc < 1) {
         Scm_Error("argument list must have at least one element: %S", args);
