@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: write.c,v 1.49 2004/11/22 23:16:05 shirok Exp $
+ *  $Id: write.c,v 1.52 2005/05/24 23:28:38 shirok Exp $
  */
 
 #include <stdio.h>
@@ -39,6 +39,7 @@
 #include "gauche.h"
 #include "gauche/port.h"
 #include "gauche/builtin-syms.h"
+#include "gauche/code.h"        /* NB: for SCM_VM_INSNP -- remove this later */
 
 static void write_walk(ScmObj obj, ScmPort *port, ScmWriteContext *ctx);
 static void write_ss(ScmObj obj, ScmPort *port, ScmWriteContext *ctx);
@@ -381,7 +382,6 @@ static void write_walk(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
         else {
             /* Now we have user-defined object.
                Call the user's print routine. */
-            ScmClass *c;
             e = Scm_HashTableGet(ht, obj);
             if (e) { e->value = SCM_TRUE; return; }
             Scm_HashTablePut(ht, obj, SCM_FALSE);
@@ -436,9 +436,6 @@ static void write_ss_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
                 else                  Scm_PutcUnsafe(ch, port);
             }
         }
-        else if (SCM_VM_INSNP(obj)) {
-            Scm__VMInsnWrite(obj, port, ctx);
-        }
         else Scm_Panic("write: got a bogus object: %08x", SCM_WORD(obj));
         return;
     }
@@ -1182,7 +1179,7 @@ void Scm_Vprintf(ScmPort *out, const char *fmt, va_list ap, int sharedp)
             case 'o':; case 'u':; case 'x':; case 'X':
                 {
                     unsigned long val = va_arg(ap, unsigned long);
-                    SCM_APPEND1(h, t, Scm_MakeIntegerFromUI(val));
+                    SCM_APPEND1(h, t, Scm_MakeIntegerU(val));
                     break;
                 }
             case 'e':; case 'E':; case 'f':; case 'g':; case 'G':
@@ -1206,7 +1203,7 @@ void Scm_Vprintf(ScmPort *out, const char *fmt, va_list ap, int sharedp)
             case 'p':
                 {
                     void *val = va_arg(ap, void *);
-                    SCM_APPEND1(h, t, Scm_MakeIntegerFromUI((unsigned long)val));
+                    SCM_APPEND1(h, t, Scm_MakeIntegerU((unsigned long)val));
                     break;
                 }
             case 'S':; case 'A':
