@@ -120,17 +120,45 @@
 ;;----------------------------------------------------------------
 (test-section "quasiquote")
 
+;; The new compiler generates constant list for much wider
+;; range of quasiquoted forms (e.g. constant numerical expressions
+;; and constant variable definitions are folded at the compile time).
+
+(define-constant quasi0 99)
+(define quasi1 101)
+(define-constant quasi2 '(a b))
+(define quasi3 '(c d))
+
 (prim-test "qq" '(1 2 3)        (lambda ()  `(1 2 3)))
 (prim-test "qq" '()             (lambda ()  `()))
+(prim-test "qq"  99             (lambda ()  `,quasi0))
+(prim-test "qq"  101            (lambda ()  `,quasi1))
 (prim-test "qq," '((1 . 2))     (lambda ()  `(,(cons 1 2))))
 (prim-test "qq," '((1 . 2) 3)   (lambda ()  `(,(cons 1 2) 3)))
+(prim-test "qq," '(99 3)        (lambda ()  `(,quasi0 3)))
+(prim-test "qq," '(3 99)        (lambda ()  `(3 ,quasi0)))
+(prim-test "qq," '(100 3)       (lambda ()  `(,(+ quasi0 1) 3)))
+(prim-test "qq," '(3 100)       (lambda ()  `(3 ,(+ quasi0 1))))
+(prim-test "qq," '(101 3)       (lambda ()  `(,quasi1 3)))
+(prim-test "qq," '(3 101)       (lambda ()  `(3 ,quasi1)))
+(prim-test "qq," '(102 3)       (lambda ()  `(,(+ quasi1 1) 3)))
+(prim-test "qq," '(3 102)       (lambda ()  `(3 ,(+ quasi1 1))))
 (prim-test "qq@" '(1 2 3 4)     (lambda ()  `(1 ,@(list 2 3) 4)))
 (prim-test "qq@" '(1 2 3 4)     (lambda ()  `(1 2 ,@(list 3 4))))
+(prim-test "qq@" '(a b c d)     (lambda ()  `(,@quasi2 ,@quasi3)))
 (prim-test "qq." '(1 2 3 4)     (lambda ()  `(1 2 . ,(list 3 4))))
+(prim-test "qq." '(a b c d)     (lambda ()  `(,@quasi2 . ,quasi3)))
 (prim-test "qq#," '#((1 . 2) 3) (lambda ()  `#(,(cons 1 2) 3)))
+(prim-test "qq#," '#(99 3)      (lambda ()  `#(,quasi0 3)))
+(prim-test "qq#," '#(100 3)     (lambda ()  `#(,(+ quasi0 1) 3)))
+(prim-test "qq#," '#(3 101)     (lambda ()  `#(3 ,quasi1)))
+(prim-test "qq#," '#(3 102)     (lambda ()  `#(3 ,(+ quasi1 1))))
 (prim-test "qq#@" '#(1 2 3 4)   (lambda ()  `#(1 ,@(list 2 3) 4)))
 (prim-test "qq#@" '#(1 2 3 4)   (lambda ()  `#(1 2 ,@(list 3 4))))
-(prim-test "qq#" '#()           (lambda ()  `#()))
+(prim-test "qq#@" '#(a b c d)   (lambda ()  `#(,@quasi2 ,@quasi3)))
+(prim-test "qq#@" '#(a b (c d)) (lambda ()  `#(,@quasi2 ,quasi3)))
+(prim-test "qq#@" '#((a b) c d) (lambda ()  `#(,quasi2  ,@quasi3)))
+(prim-test "qq#"  '#()          (lambda ()  `#()))
 (prim-test "qq#@" '#()          (lambda ()  `#(,@(list))))
 
 (prim-test "qq@@" '(1 2 1 2)    (lambda ()  `(,@(list 1 2) ,@(list 1 2))))
@@ -142,8 +170,12 @@
       (lambda ()  `(,@(list 1 2) ,@(list 1 2) . a)))
 (prim-test "qq@." '(1 2 1 2 1 . 2)
       (lambda ()  `(,@(list 1 2) ,@(list 1 2) . ,(cons 1 2))))
+(prim-test "qq@." '(1 2 1 2 a b)
+      (lambda ()  `(,@(list 1 2) ,@(list 1 2) . ,quasi2)))
 (prim-test "qq@." '(1 2 1 2 a 1 . 2)
       (lambda ()  `(,@(list 1 2) ,@(list 1 2) a . ,(cons 1 2))))
+(prim-test "qq@." '(1 2 1 2 a c d)
+      (lambda ()  `(,@(list 1 2) ,@(list 1 2) a . ,quasi3)))
 
 (prim-test "qq#@@" '#(1 2 1 2)    (lambda ()  `#(,@(list 1 2) ,@(list 1 2))))
 (prim-test "qq#@@" '#(1 2 a 1 2)  (lambda ()  `#(,@(list 1 2) a ,@(list 1 2))))
@@ -151,10 +183,28 @@
 (prim-test "qq#@@" '#(1 2 1 2 a)  (lambda ()  `#(,@(list 1 2) ,@(list 1 2) a)))
 (prim-test "qq#@@" '#(1 2 1 2 a b) (lambda () `#(,@(list 1 2) ,@(list 1 2) a b)))
 
-(prim-test "qqq"   '(1 `(1 ,2 ,3) 1)  (lambda ()  `(1 `(1 ,2 ,,(+ 1 2)) 1)))
-(prim-test "qqq"   '(1 `(1 ,@2 ,@(1 2))) (lambda () `(1 `(1 ,@2 ,@,(list 1 2)))))
-(prim-test "qqq#"  '#(1 `(1 ,2 ,3) 1)  (lambda ()  `#(1 `(1 ,2 ,,(+ 1 2)) 1)))
-(prim-test "qqq#"  '#(1 `(1 ,@2 ,@(1 2))) (lambda () `#(1 `(1 ,@2 ,@,(list 1 2)))))
+(prim-test "qqq"   '(1 `(1 ,2 ,3) 1)
+           (lambda ()  `(1 `(1 ,2 ,,(+ 1 2)) 1)))
+(prim-test "qqq"   '(1 `(1 ,99 ,101) 1)
+           (lambda ()  `(1 `(1 ,,quasi0 ,,quasi1) 1)))
+(prim-test "qqq"   '(1 `(1 ,@2 ,@(1 2)))
+           (lambda () `(1 `(1 ,@2 ,@,(list 1 2)))))
+(prim-test "qqq"   '(1 `(1 ,@(a b) ,@(c d)))
+           (lambda () `(1 `(1 ,@,quasi2 ,@,quasi3))))
+(prim-test "qqq"   '(1 `(1 ,(a b x) ,(y c d)))
+           (lambda () `(1 `(1 ,(,@quasi2 x) ,(y ,@quasi3)))))
+(prim-test "qqq#"  '#(1 `(1 ,2 ,3) 1)
+           (lambda ()  `#(1 `(1 ,2 ,,(+ 1 2)) 1)))
+(prim-test "qqq#"  '#(1 `(1 ,99 ,101) 1)
+           (lambda ()  `#(1 `(1 ,,quasi0 ,,quasi1) 1)))
+(prim-test "qqq#"  '#(1 `(1 ,@2 ,@(1 2)))
+           (lambda () `#(1 `(1 ,@2 ,@,(list 1 2)))))
+(prim-test "qqq#"  '#(1 `(1 ,@(a b) ,@(c d)))
+           (lambda () `#(1 `(1 ,@,quasi2 ,@,quasi3))))
+(prim-test "qqq#"  '#(1 `(1 ,(a b x) ,(y c d)))
+           (lambda () `#(1 `(1 ,(,@quasi2 x) ,(y ,@quasi3)))))
+(prim-test "qqq#"  '(1 `#(1 ,(a b x) ,(y c d)))
+           (lambda () `(1 `#(1 ,(,@quasi2 x) ,(y ,@quasi3)))))
 
 ;;----------------------------------------------------------------
 (test-section "multiple values")
