@@ -2,7 +2,7 @@
 ;; testing regexp
 ;;
 
-;; $Id: regexp.scm,v 1.19 2004/09/12 21:42:40 shirok Exp $
+;; $Id: regexp.scm,v 1.20 2005/05/24 07:46:14 shirok Exp $
 
 (use gauche.test)
 (use srfi-1)
@@ -10,410 +10,410 @@
 (test-start "regexp")
 
 ;; Some test examples are taken from the test suite of Henry Spencer's
-;; regexp package.
+;; regexp package and PCRE.
 
-(define (match&list rx input nmatch)
+(define (match&list rx input)
   (cond ((rxmatch rx input)
          => (lambda (match)
               (map (lambda (n) (rxmatch-substring match n))
-                   (iota nmatch))))
+                   (iota (rxmatch-num-matches match)))))
         (else #f)))
 
 ;;-------------------------------------------------------------------------
 (test-section "basics")
 
 (test* "a" '("a")
-       (match&list #/a/ "a" 1))
+       (match&list #/a/ "a"))
 (test* "a" #f
-       (match&list #/a/ "A" 1))
+       (match&list #/a/ "A"))
 (test* "a" '("a")
-       (match&list #/a/ "ba" 1))
+       (match&list #/a/ "ba"))
 (test* "a" '("a")
-       (match&list #/a/ "bac" 1))
+       (match&list #/a/ "bac"))
 (test* "a" #f      ;; input null str
-       (match&list #/a/ "" 1))
+       (match&list #/a/ ""))
 (test* "a" '("a")  ;; input includes NUL character
-       (match&list #/a/ (string (integer->char 0) #\a #\b) 1))
+       (match&list #/a/ (string (integer->char 0) #\a #\b)))
 (test* "abc" '("abc")
-       (match&list #/abc/ "abc" 1))
+       (match&list #/abc/ "abc"))
 (test* "abc" #f
-       (match&list #/abc/ "abbc" 1))
+       (match&list #/abc/ "abbc"))
 (test* "abc" '("abc")
-       (match&list #/abc/ "babcd" 1))
+       (match&list #/abc/ "babcd"))
 (test* "abc|de" '("abc")
-       (match&list #/abc|de/ "dabce" 1))
+       (match&list #/abc|de/ "dabce"))
 (test* "abc|de" '("de")
-       (match&list #/abc|de/ "abdec" 1))
+       (match&list #/abc|de/ "abdec"))
 (test* "abc|de" #f
-       (match&list #/abc|de/ "abe" 1))
+       (match&list #/abc|de/ "abe"))
 (test* "a|b|c" '("a")
-       (match&list #/a|b|c/ "abc" 1))
+       (match&list #/a|b|c/ "abc"))
 (test* "a|b|c" '("b")
-       (match&list #/a|b|c/ "bac" 1))
+       (match&list #/a|b|c/ "bac"))
 (test* "a|b|c" #f
-       (match&list #/a|b|c/ "def" 1))
+       (match&list #/a|b|c/ "def"))
 (test* "|abc" '("")
-       (match&list #/|abc/ "abc" 1))
+       (match&list #/|abc/ "abc"))
 (test* "abc|" '("abc")
-       (match&list #/abc|/ "abc" 1))
+       (match&list #/abc|/ "abc"))
 (test* "abc|" '("")
-       (match&list #/abc|/ "abd" 1))
+       (match&list #/abc|/ "abd"))
 
 ;;-------------------------------------------------------------------------
 (test-section "parens")
 
 (test* "a(b)c" '("abc" "b")
-       (match&list #/a(b)c/ "abc" 2))
+       (match&list #/a(b)c/ "abc"))
 (test* "a((b)(c))" '("abc" "bc" "b" "c")
-       (match&list #/a((b)(c))/ "abc" 4))
+       (match&list #/a((b)(c))/ "abc"))
 (test* "a((((b))))c" '("abc" "b" "b" "b" "b")
-       (match&list #/a((((b))))c/ "abc" 5))
+       (match&list #/a((((b))))c/ "abc"))
 (test* "a((((b))))c" '#f
-       (match&list #/a((((b))))c/ "a(b)c" 5))
+       (match&list #/a((((b))))c/ "a(b)c"))
 (test* "a\\(" '("a(")
-       (match&list #/a\(/ "a(" 1))
+       (match&list #/a\(/ "a("))
 (test* "a()b" '("ab" "")
-       (match&list #/a()b/ "ab" 2))
+       (match&list #/a()b/ "ab"))
 (test* "a()()b" '("ab" "" "")
-       (match&list #/a()()b/ "ab" 3))
+       (match&list #/a()()b/ "ab"))
 (test* "(we|wee|week|frob)(knights|night|day)"
        '("weeknights" "wee" "knights")
        (match&list #/(we|wee|week|frob)(knights|night|day)/
-                   "weeknights" 3))
+                   "weeknights"))
 (test* "aa|(bb)|cc" '("aa" #f)
-       (match&list #/aa|(bb)|cc/ "aabb" 2))
+       (match&list #/aa|(bb)|cc/ "aabb"))
 (test* "aa|(bb)|cc" '("bb" "bb")
-       (match&list #/aa|(bb)|cc/ "abbaa" 2))
+       (match&list #/aa|(bb)|cc/ "abbaa"))
 (test* "aa|(bb)|cc" '("cc" #f)
-       (match&list #/aa|(bb)|cc/ "bccaa" 2))
+       (match&list #/aa|(bb)|cc/ "bccaa"))
 (test* "aa|a(b)|cc" '("ab" "b")
-       (match&list #/aa|a(b)|cc/ "abaab" 2))
+       (match&list #/aa|a(b)|cc/ "abaab"))
 (test* "aa|a(b)" '("ab" "b")
-       (match&list #/aa|a(b)/ "abaab" 2))
+       (match&list #/aa|a(b)/ "abaab"))
 (test* "aa|(a(b))|ac" '("ab" "ab" "b")
-       (match&list #/aa|(a(b))|cc/ "abaabcc" 3))
+       (match&list #/aa|(a(b))|cc/ "abaabcc"))
 (test* "(ab)|ac" '("ab" "ab")
-       (match&list #/(ab)|ac/ "aaaabcc" 2))
+       (match&list #/(ab)|ac/ "aaaabcc"))
 (test* "(a(b))|ac" '("ab" "ab" "b")
-       (match&list #/(a(b))|ac/ "abaabcc" 3))
+       (match&list #/(a(b))|ac/ "abaabcc"))
 (test* "ab|(ac)" '("ab" #f)
-       (match&list #/ab|(ac)/ "aaaabcc" 2))
+       (match&list #/ab|(ac)/ "aaaabcc"))
 (test* "ab|(ac)" '("ac" "ac")
-       (match&list #/ab|(ac)/ "aaaacbc" 2))
+       (match&list #/ab|(ac)/ "aaaacbc"))
 (test* "aa|(ab|(ac))|ad" '("ac" "ac" "ac")
-       (match&list #/aa|(ab|(ac))|ad/ "cac" 3))
+       (match&list #/aa|(ab|(ac))|ad/ "cac"))
 (test* "(aa|(a(b)|a(c))|ad)" '("ac" "ac" "ac" #f "c")
-       (match&list #/(aa|(a(b)|a(c))|ad)/ "cac" 5))
+       (match&list #/(aa|(a(b)|a(c))|ad)/ "cac"))
 (test* "b|()|a" '("" "")
-       (match&list #/b|()|a/ "cac" 2))
+       (match&list #/b|()|a/ "cac"))
 
 ;;-------------------------------------------------------------------------
 (test-section "simple meta")
 
 (test* "a.c" '("abc")
-       (match&list #/a.c/ "abc" 1))
+       (match&list #/a.c/ "abc"))
 (test* "a.." '("abc")
-       (match&list #/a../ "abc" 1))
+       (match&list #/a../ "abc"))
 (test* "a.." #f
-       (match&list #/a../ "ab" 1))
+       (match&list #/a../ "ab"))
 (test* "..." #f
-       (match&list #/.../ "ab" 1))
+       (match&list #/.../ "ab"))
 (test* "." '("a")
-       (match&list #/./ "abc" 1))
+       (match&list #/./ "abc"))
 (test* "." #f
-       (match&list #/./ "" 1))
+       (match&list #/./ ""))
 
 ;;-------------------------------------------------------------------------
 (test-section "anchors")
 
 (test* "^abc" '("abc")
-       (match&list #/^abc/ "abcd" 1))
+       (match&list #/^abc/ "abcd"))
 (test* "^abc" #f
-       (match&list #/^abc/ "aabcd" 1))
+       (match&list #/^abc/ "aabcd"))
 (test* "^^" '("^")
-       (match&list #/^^/ "^^abc" 1))
+       (match&list #/^^/ "^^abc"))
 (test* "^^" #f
-       (match&list #/^^/ "a^^c" 1))
+       (match&list #/^^/ "a^^c"))
 (test* "^abc|def" '("abc")
-       (match&list #/^abc|def/ "abc" 1))
+       (match&list #/^abc|def/ "abc"))
 (test* "^abc|def" #f
-       (match&list #/^abc|def/ "zabc" 1))
+       (match&list #/^abc|def/ "zabc"))
 (test* "^abc|def" '("def")
-       (match&list #/^abc|def/ "zabcdef" 1))
+       (match&list #/^abc|def/ "zabcdef"))
 (test* "abc|^def" '("def")
-       (match&list #/abc|^def/ "defabc" 1))
+       (match&list #/abc|^def/ "defabc"))
 (test* "abc|^def" '("abc")
-       (match&list #/abc|^def/ "abcdef" 1))
+       (match&list #/abc|^def/ "abcdef"))
 (test* "abc|^def" '("def")
-       (match&list #/abc|^def/ "defabbc" 1))
+       (match&list #/abc|^def/ "defabbc"))
 (test* "abc|^def" #f
-       (match&list #/abc|^def/ "adefbc" 1))
+       (match&list #/abc|^def/ "adefbc"))
 (test* "^(abc|def)" '("abc" "abc")
-       (match&list #/^(abc|def)/ "abc" 2))
+       (match&list #/^(abc|def)/ "abc"))
 (test* "^(abc|def)" #f
-       (match&list #/^(abc|def)/ "aabc" 2))
+       (match&list #/^(abc|def)/ "aabc"))
 (test* "(^abc|def)" '("abc" "abc")
-       (match&list #/(^abc|def)/ "abcdef" 2))
+       (match&list #/(^abc|def)/ "abcdef"))
 (test* "(^abc|def)" '("def" "def")
-       (match&list #/(^abc|def)/ "^abcdef" 2))
+       (match&list #/(^abc|def)/ "^abcdef"))
 (test* "a(^bc|def)" '("a^bc" "^bc")
-       (match&list #/a(^bc|def)/ "a^bcdef" 2))
+       (match&list #/a(^bc|def)/ "a^bcdef"))
 (test* "a(^bc|def)" #f
-       (match&list #/a(^bc|def)/ "abcdef" 2))
+       (match&list #/a(^bc|def)/ "abcdef"))
 (test* "^" '("")
-       (match&list #/^/ "hoge" 1))
+       (match&list #/^/ "hoge"))
 (test* "$" '("")
-       (match&list #/$/ "hoge" 1))
+       (match&list #/$/ "hoge"))
 (test* "abc$" '("abc")
-       (match&list #/abc$/ "bcabc" 1))
+       (match&list #/abc$/ "bcabc"))
 (test* "abc$" #f
-       (match&list #/abc$/ "abcab" 1))
+       (match&list #/abc$/ "abcab"))
 (test* "^abc$" '("abc")
-       (match&list #/^abc$/ "abc" 1))
+       (match&list #/^abc$/ "abc"))
 (test* "abc$$" #f
-       (match&list #/abc$$/ "abc" 1))
+       (match&list #/abc$$/ "abc"))
 (test* "abc$$" '("abc$")
-       (match&list #/abc$$/ "abc$" 1))
+       (match&list #/abc$$/ "abc$"))
 (test* "$$" '("$")
-       (match&list #/$$/ "abc$" 1))
+       (match&list #/$$/ "abc$"))
 (test* "^$" '("")
-       (match&list #/^$/ "" 1))
+       (match&list #/^$/ ""))
 (test* "^$" #f
-       (match&list #/^$/ "a" 1))
+       (match&list #/^$/ "a"))
 (test* "^^$$" '("^$")
-       (match&list #/^^$$/ "^$" 1))
+       (match&list #/^^$$/ "^$"))
 (test* "abc$|def" '("abc")
-       (match&list #/abc$|def/ "abc" 1))
+       (match&list #/abc$|def/ "abc"))
 (test* "abc$|def" '("def")
-       (match&list #/abc$|def/ "defabc" 1))
+       (match&list #/abc$|def/ "defabc"))
 (test* "^abc|def$" '("abc")
-       (match&list #/^abc|def$/ "abcdef" 1))
+       (match&list #/^abc|def$/ "abcdef"))
 (test* "^abc|def$" #f
-       (match&list #/^abc|def$/ "defabc" 1))
+       (match&list #/^abc|def$/ "defabc"))
 (test* "^abc|def$" #f
-       (match&list #/^abc|def$/ "defabc" 1))
+       (match&list #/^abc|def$/ "defabc"))
 (test* "(^abc|def$)" '("def" "def")
-       (match&list #/(^abc|def$)/ "aaadef" 2))
+       (match&list #/(^abc|def$)/ "aaadef"))
 (test* "(^abc|def$)$" #f
-       (match&list #/(^abc|def$)$/ "aaadef" 1))
-(test* "(^abc|def$)$" '("def$")
-       (match&list #/(^abc|def$)$/ "aaadef$" 1))
+       (match&list #/(^abc|def$)$/ "aaadef"))
+(test* "(^abc|def$)$" '("def$" "def$")
+       (match&list #/(^abc|def$)$/ "aaadef$"))
 (test* "(abc$|def)$" #f
-       (match&list #/(abc$|def)$/ "aaabc" 1))
-(test* "(abc$|def)$" '("abc$")
-       (match&list #/(abc$|def)$/ "aaabc$" 1))
+       (match&list #/(abc$|def)$/ "aaabc"))
+(test* "(abc$|def)$" '("abc$" "abc$")
+       (match&list #/(abc$|def)$/ "aaabc$"))
 (test* "a$b" '("a$b")
-       (match&list #/a$b/ "aa$bb" 1))
+       (match&list #/a$b/ "aa$bb"))
 (test* "ab\\$" '("ab$")
-       (match&list #/ab\$/ "ab$cd" 1))
+       (match&list #/ab\$/ "ab$cd"))
 
 ;;-------------------------------------------------------------------------
 (test-section "backslash escape")
 
 (test* "a\\*c" '("a*c")
-       (match&list #/a\*c/ "a*c" 1))
+       (match&list #/a\*c/ "a*c"))
 (test* "a\\.c" '("a.c")
-       (match&list #/a\.c/ "a.c" 1))
+       (match&list #/a\.c/ "a.c"))
 (test* "a\\.c" #f
-       (match&list #/a\.c/ "abc" 1))
+       (match&list #/a\.c/ "abc"))
 (test* "a\\\\b" '("a\\b")
-       (match&list #/a\\b/ "a\\b" 1))
+       (match&list #/a\\b/ "a\\b"))
 (test* "a\\\\\\*b" '("a\\*b")
-       (match&list #/a\\\*b/ "a\\*b" 1))
+       (match&list #/a\\\*b/ "a\\*b"))
 (test* "a\\jc" '("ajc")
-       (match&list #/a\jc/ "ajc" 1))
+       (match&list #/a\jc/ "ajc"))
 (test* "a\\\\bc" '("a\\bc")
-       (match&list #/a\\bc/ "a\\bc" 1))
+       (match&list #/a\\bc/ "a\\bc"))
 (test* "a\\[b" '("a[b")
-       (match&list #/a\[b/ "a[b" 1))
+       (match&list #/a\[b/ "a[b"))
 
 ;;-------------------------------------------------------------------------
 (test-section "word boundary")
 
 (test* ".z\\b" '("oz")
-       (match&list #/.z\b/ "bzbazoz ize" 1))
+       (match&list #/.z\b/ "bzbazoz ize"))
 (test* "\\b.z" '("iz")
-       (match&list #/\b.z/ "brzbazoz ize" 1))
+       (match&list #/\b.z/ "brzbazoz ize"))
 (test* ".z\\B" '("iz")
-       (match&list #/.z\B/ "bz baz oz ize" 1))
+       (match&list #/.z\B/ "bz baz oz ize"))
 (test* "\\B.z" '("az")
-       (match&list #/\B.z/ "bz baz oz ize" 1))
+       (match&list #/\B.z/ "bz baz oz ize"))
 
 ;;-------------------------------------------------------------------------
 (test-section "repetitions")
 
 (test* "ab*c" '("abc")
-       (match&list #/ab*c/ "abc" 1))
+       (match&list #/ab*c/ "abc"))
 (test* "ab*c" '("ac")
-       (match&list #/ab*c/ "ac" 1))
+       (match&list #/ab*c/ "ac"))
 (test* "ab*c" '("abbbc")
-       (match&list #/ab*c/ "abbbc" 1))
+       (match&list #/ab*c/ "abbbc"))
 (test* "ab*c" '("abbc")
-       (match&list #/ab*c/ "abbabaabbc" 1))
+       (match&list #/ab*c/ "abbabaabbc"))
 (test* "ab+c" '("abc")
-       (match&list #/ab+c/ "abc" 1))
+       (match&list #/ab+c/ "abc"))
 (test* "ab+c" '("abbc")
-       (match&list #/ab+c/ "abbc" 1))
+       (match&list #/ab+c/ "abbc"))
 (test* "ab+c" '("abbc")
-       (match&list #/ab+c/ "abbabaabbc" 1))
+       (match&list #/ab+c/ "abbabaabbc"))
 (test* "ab?c" '("abc")
-       (match&list #/ab?c/ "abc" 1))
+       (match&list #/ab?c/ "abc"))
 (test* "ab?c" '("ac")
-       (match&list #/ab?c/ "abbaac" 1))
+       (match&list #/ab?c/ "abbaac"))
 (test* "a.*c" '("abc")
-       (match&list #/a.*c/ "abc" 1))
+       (match&list #/a.*c/ "abc"))
 (test* "a.*c" '("aabcabcabcabcc")
-       (match&list #/a.*c/ "zaabcabcabcabcczab" 1))
+       (match&list #/a.*c/ "zaabcabcabcabcczab"))
 (test* "a(b*|c)d" '("abbd" "bb")
-       (match&list #/a(b*|c)d/ "abbd" 2))
+       (match&list #/a(b*|c)d/ "abbd"))
 (test* "a(b*|c)d" '("ad" "")
-       (match&list #/a(b*|c)d/ "ad" 2))
+       (match&list #/a(b*|c)d/ "ad"))
 (test* "a(b*|c)d" '("acd" "c")
-       (match&list #/a(b*|c)d/ "acd" 2))
+       (match&list #/a(b*|c)d/ "acd"))
 (test* "a(b*|c)d" #f
-       (match&list #/a(b*|c)d/ "abcd" 2))
+       (match&list #/a(b*|c)d/ "abcd"))
 (test* "a.*c" '("ac")
-       (match&list #/a.*c/ "bacbababbbbadbaba" 1))
+       (match&list #/a.*c/ "bacbababbbbadbaba"))
 (test* "a.*c" #f
-       (match&list #/a.*c/ "abaaaabababbadbabdba" 1))
+       (match&list #/a.*c/ "abaaaabababbadbabdba"))
 
 ;;-------------------------------------------------------------------------
 (test-section "repetitions (non-greedy)")
 
 (test* "ab*?." '("ab")
-       (match&list #/ab*?./ "abc" 1))
+       (match&list #/ab*?./ "abc"))
 (test* "ab*?." '("ac")
-       (match&list #/ab*?./ "ac" 1))
+       (match&list #/ab*?./ "ac"))
 (test* "a.*?c" '("abbbc")
-       (match&list #/a.*?c/ "abbbc" 1))
+       (match&list #/a.*?c/ "abbbc"))
 (test* "a.*?a" '("abba")
-       (match&list #/a.*?a/ "abbabaabbc" 1))
+       (match&list #/a.*?a/ "abbabaabbc"))
 (test* "<.*?>" '("<tag1>")
-       (match&list #/<.*?>/ "<tag1><tag2><tag3>" 1))
+       (match&list #/<.*?>/ "<tag1><tag2><tag3>"))
 
 (test* "ab+?." '("abc")
-       (match&list #/ab+?./ "abc" 1))
+       (match&list #/ab+?./ "abc"))
 (test* "ab+?." '("abb")
-       (match&list #/ab+?./ "abbc" 1))
+       (match&list #/ab+?./ "abbc"))
 (test* "a.+?a" '("abba")
-       (match&list #/a.+?a/ "abbabaabbc" 1))
+       (match&list #/a.+?a/ "abbabaabbc"))
 (test* "<.+?>" '("<><tag1>")
-       (match&list #/<.+?>/ " <><tag1><tag2>" 1))
+       (match&list #/<.+?>/ " <><tag1><tag2>"))
 
 (test* "ab??c" '("abc")
-       (match&list #/ab??c/ "abc" 1))
+       (match&list #/ab??c/ "abc"))
 (test* "ab??c" '("ac")
-       (match&list #/ab??c/ "abbaac" 1))
+       (match&list #/ab??c/ "abbaac"))
 (test* "ab??." '("ab")
-       (match&list #/ab??./ "abbaac" 1))
-(test* "a(hoge)??hoge" '("ahoge")
-       (match&list #/a(hoge)??hoge/ "ahogehoge" 1))
+       (match&list #/ab??./ "abbaac"))
+(test* "a(hoge)??hoge" '("ahoge" #f)
+       (match&list #/a(hoge)??hoge/ "ahogehoge"))
 
 ;;-------------------------------------------------------------------------
 (test-section "character class")
 
 (test* "a[bc]d" '("abd")
-       (match&list #/a[bc]d/ "abd" 1))
+       (match&list #/a[bc]d/ "abd"))
 (test* "a[bc]d" '("acd")
-       (match&list #/a[bc]d/ "acd" 1))
+       (match&list #/a[bc]d/ "acd"))
 (test* "a[bc]d" #f
-       (match&list #/a[bc]d/ "aed" 1))
+       (match&list #/a[bc]d/ "aed"))
 (test* "a[a-z]d" '("aed")
-       (match&list #/a[a-z]d/ "aed" 1))
+       (match&list #/a[a-z]d/ "aed"))
 (test* "a[a-z]d" #f
-       (match&list #/a[a-z]d/ "aEd" 1))
+       (match&list #/a[a-z]d/ "aEd"))
 (test* "a[]]d" '("a]d")
-       (match&list #/a[]]d/ "a]d" 1))
+       (match&list #/a[]]d/ "a]d"))
 (test* "a[]-]d" '("a-d")
-       (match&list #/a[]-]d/ "a-d" 1))
+       (match&list #/a[]-]d/ "a-d"))
 (test* "a[]-^]d" #f
-       (match&list #/a[]-^]d/ "a-d" 1))
+       (match&list #/a[]-^]d/ "a-d"))
 (test* "a[]-^]d" '("a]d")
-       (match&list #/a[]-^]d/ "a]d" 1))
+       (match&list #/a[]-^]d/ "a]d"))
 (test* "a[a-z-]d" '("a-d")
-       (match&list #/a[a-z-]d/ "a-d" 1))
+       (match&list #/a[a-z-]d/ "a-d"))
 (test* "a[a-z-]d" '("afd")
-       (match&list #/a[a-z-]d/ "afd" 1))
+       (match&list #/a[a-z-]d/ "afd"))
 (test* "a[az-]d" '("a-d")
-       (match&list #/a[az-]d/ "a-d" 1))
+       (match&list #/a[az-]d/ "a-d"))
 (test* "a[a-]d" '("a-d")
-       (match&list #/a[a-]d/ "a-d" 1))
+       (match&list #/a[a-]d/ "a-d"))
 (test* "a[az-]d" #f
-       (match&list #/a[az-]d/ "afd" 1))
+       (match&list #/a[az-]d/ "afd"))
 (test* "a[az-]d" '("azd")
-       (match&list #/a[az-]d/ "azd" 1))
+       (match&list #/a[az-]d/ "azd"))
 (test* "a[^ab]c" '("acc")
-       (match&list #/a[^ab]c/ "abacc" 1))
+       (match&list #/a[^ab]c/ "abacc"))
 (test* "a[^]]c" '("abc")
-       (match&list #/a[^]]c/ "abc" 1))
+       (match&list #/a[^]]c/ "abc"))
 (test* "a[^]]c" #f
-       (match&list #/a[^]]c/ "a]c" 1))
+       (match&list #/a[^]]c/ "a]c"))
 (test* "a[^^]c" '("abc")
-       (match&list #/a[^^]c/ "abc" 1))
+       (match&list #/a[^^]c/ "abc"))
 (test* "a[^^]c" #f
-       (match&list #/a[^^]c/ "a^c" 1))
+       (match&list #/a[^^]c/ "a^c"))
 (test* "a[Bc]*d" '("aBccBd")
-       (match&list #/a[Bc]*d/ "aBccBd" 1))
+       (match&list #/a[Bc]*d/ "aBccBd"))
 (test* "[a]b[c]" '("abc")
-       (match&list #/[a]b[c]/ "abc" 1))
+       (match&list #/[a]b[c]/ "abc"))
 (test* "[abc]b[abc]" '("abc")
-       (match&list #/[abc]b[abc]/ "abc" 1))
+       (match&list #/[abc]b[abc]/ "abc"))
 (test* "a[bc]d" '("abd")
-       (match&list #/a[bc]d/ "xyzaaabcaababdacd" 1))
+       (match&list #/a[bc]d/ "xyzaaabcaababdacd"))
 (test* "a[ab][ab][ab][ab][ab][ab][ab][ab][ab][ab][ab][ab][ab][ab][ab][ab][ab][ab][ab][ab](wee|week)(knights|night)"
        '("aaaaabaaaabaaaabaaaabweeknights" "wee" "knights")
        (match&list #/a[ab][ab][ab][ab][ab][ab][ab][ab][ab][ab][ab][ab][ab][ab][ab][ab][ab][ab][ab][ab](wee|week)(knights|night)/
-                   "aaaaabaaaabaaaabaaaabweeknights" 3))
+                   "aaaaabaaaabaaaabaaaabweeknights"))
 (test* "[ab][cd][ef][gh][ij][kl][mn]"
        '("acegikm")
        (match&list #/[ab][cd][ef][gh][ij][kl][mn]/
-                   "xacegikmoq" 1))
+                   "xacegikmoq"))
 (test* "[ab][cd][ef][gh][ij][kl][mn][op]"
        '("acegikmo")
        (match&list #/[ab][cd][ef][gh][ij][kl][mn][op]/
-                   "xacegikmoq" 1))
+                   "xacegikmoq"))
 (test* "[ab][cd][ef][gh][ij][kl][mn][op][qr]"
        '("acegikmoq")
        (match&list #/[ab][cd][ef][gh][ij][kl][mn][op][qr]/
-                   "xacegikmoqy" 1))
+                   "xacegikmoqy"))
 (test* "[ab][cd][ef][gh][ij][kl][mn][op][q]"
        '("acegikmoq")
        (match&list #/[ab][cd][ef][gh][ij][kl][mn][op][q]/
-                   "xacegikmoqy" 1))
+                   "xacegikmoqy"))
 
 ;; this tests optimizer
 (test* "[^a]*." '("b")
-       (match&list #/[^a]*./ "b" 1))
+       (match&list #/[^a]*./ "b"))
 
-(test* "\\s" '(" ")  (match&list #/\s/ "  " 1))
-(test* "\\s" '("  ")  (match&list #/\s\s/ "  " 1))
-(test* "\\s" '("\t")  (match&list #/\s/ "\t " 1))
-(test* "\\s" #f  (match&list #/\s\s/ "\\s" 1))
-(test* "\\\\s" '("\\s ")  (match&list #/\\s\s/ "\\s " 1))
+(test* "\\s" '(" ")  (match&list #/\s/ "  "))
+(test* "\\s" '("  ")  (match&list #/\s\s/ "  "))
+(test* "\\s" '("\t")  (match&list #/\s/ "\t "))
+(test* "\\s" #f  (match&list #/\s\s/ "\\s"))
+(test* "\\\\s" '("\\s ")  (match&list #/\\s\s/ "\\s "))
 (test* "\\s\\S" '("\txyz   " "\t" "xyz" "   ")
-       (match&list #/(\s*)(\S+)(\s*)/ "\txyz   abc" 4))
+       (match&list #/(\s*)(\S+)(\s*)/ "\txyz   abc"))
 (test* "\\d\\D" '("1234) 5678-9012" "1234" ") " "5678" "-" "9012")
        (match&list #/(\d+)(\D+)(\d+)(\D+)(\d+)/
-                   " (1234) 5678-9012 " 6))
+                   " (1234) 5678-9012 "))
 (test* "\\w\\W" '("three o" "three" " " "o")
        (match&list #/(\w+)(\W+)(\w+)/
-                   "three o'clock" 4))
+                   "three o'clock"))
 
-(test* "[ab\\sc]+" '(" ba ") (match&list #/[ab\sc]+/ "d ba e" 1))
-(test* "[\\sa-c]+" '(" ba ") (match&list #/[\sabc]+/ "d ba e" 1))
-(test* "[\\S\\t]+" '("\tab") (match&list #/[\S\t]+/ "\tab cd" 1))
+(test* "[ab\\sc]+" '(" ba ") (match&list #/[ab\sc]+/ "d ba e"))
+(test* "[\\sa-c]+" '(" ba ") (match&list #/[\sabc]+/ "d ba e"))
+(test* "[\\S\\t]+" '("\tab") (match&list #/[\S\t]+/ "\tab cd"))
 (test* "[\\s\\d]+" '(" 1 2 3 ")
-       (match&list #/[\s\d]+/ "a 1 2 3 b " 1))
+       (match&list #/[\s\d]+/ "a 1 2 3 b "))
 (test* "[\\s\\D]+" '("a ")
-       (match&list #/[\s\D]+/ "a 1 2 3 b " 1))
+       (match&list #/[\s\D]+/ "a 1 2 3 b "))
 
 ;; this tests optimizer
 (test* "^\\[?([^\\]]*)\\]?:(\\d+)$" '("127.0.0.1:80" "127.0.0.1" "80")
-       (match&list #/^\[?([^\]]*)\]?:(\d+)$/ "127.0.0.1:80" 3))
+       (match&list #/^\[?([^\]]*)\]?:(\d+)$/ "127.0.0.1:80"))
 (test* "^\\[?([^\\]]*)\\]?:(\\d+)$" '("[127.0.0.1:80" "127.0.0.1" "80")
-       (match&list #/^\[?([^\]]*)\]?:(\d+)$/ "[127.0.0.1:80" 3))
+       (match&list #/^\[?([^\]]*)\]?:(\d+)$/ "[127.0.0.1:80"))
 (test* "^\\[?([^\\]]*)\\]?:(\\d+)$" '("[127.0.0.1]:80" "127.0.0.1" "80")
-       (match&list #/^\[?([^\]]*)\]?:(\d+)$/ "[127.0.0.1]:80" 3))
+       (match&list #/^\[?([^\]]*)\]?:(\d+)$/ "[127.0.0.1]:80"))
 
 ;;-------------------------------------------------------------------------
 (test-section "{n,m}")
@@ -433,17 +433,17 @@
 (test* "(ab){b,c}" "def"       (rxmatch-after (#/(ab){b,c}/ "zab{b,c}def")))
 
 (test* "(\\d{2})(\\d{2})" '("1234" "12" "34")
-       (match&list #/(\d{2})(\d{2})/ "a12345b" 3))
+       (match&list #/(\d{2})(\d{2})/ "a12345b"))
 (test* "(\\d{2,})(\\d{2,})" '("12345" "123" "45")
-       (match&list #/(\d{2,})(\d{2,})/ "a12345b" 3))
+       (match&list #/(\d{2,})(\d{2,})/ "a12345b"))
 (test* "(\\d{2})(\\d{2,})" '("12345" "12" "345")
-       (match&list #/(\d{2})(\d{2,})/ "a12345b" 3))
+       (match&list #/(\d{2})(\d{2,})/ "a12345b"))
 (test* "(\\d{1,3})(\\d{2,})" '("1234" "12" "34")
-       (match&list #/(\d{1,3})(\d{2,})/ "a1234b" 3))
+       (match&list #/(\d{1,3})(\d{2,})/ "a1234b"))
 (test* "(\\d{1,3})(\\d{0,2})" '("1234" "123" "4")
-       (match&list #/(\d{1,3})(\d{0,2})/ "a1234b" 3))
-(test* "(\\d{2}){2}" '("1234")
-       (match&list #/(\d{2}){2}/ "a12345b" 1))
+       (match&list #/(\d{1,3})(\d{0,2})/ "a1234b"))
+(test* "(\\d{2}){2}" '("1234" "12")
+       (match&list #/(\d{2}){2}/ "a12345b"))
 
 (test* "{2}" *test-error*  (regexp? (string->regexp "{2}")))
 (test* "{z}" #t            (regexp? (string->regexp "{z}")))
@@ -466,15 +466,15 @@
 (test* "(ab){2,3}?" "abba"      (rxmatch-after (#/(ab){2,3}?/ "babbabababba")))
 
 (test* "(\\d{2,}?)(\\d{2,}?)" '("1234" "12" "34")
-       (match&list #/(\d{2,}?)(\d{2,}?)/ "a12345b" 3))
+       (match&list #/(\d{2,}?)(\d{2,}?)/ "a12345b"))
 (test* "(\\d{2,})(\\d{2,}?)" '("12345" "123" "45")
-       (match&list #/(\d{2,})(\d{2,}?)/ "a12345b" 3))
+       (match&list #/(\d{2,})(\d{2,}?)/ "a12345b"))
 (test* "(\\d{2,}?)(\\d{2,})" '("12345" "12" "345")
-       (match&list #/(\d{2,}?)(\d{2,})/ "a12345b" 3))
+       (match&list #/(\d{2,}?)(\d{2,})/ "a12345b"))
 (test* "(\\d{1,3}?)(\\d{2,}?)" '("123" "1" "23")
-       (match&list #/(\d{1,3}?)(\d{2,}?)/ "a1234b" 3))
+       (match&list #/(\d{1,3}?)(\d{2,}?)/ "a1234b"))
 (test* "(\\d{1,3}?)(\\d{0,2}?)" '("1" "1" "")
-       (match&list #/(\d{1,3}?)(\d{0,2}?)/ "a1234b" 3))
+       (match&list #/(\d{1,3}?)(\d{0,2}?)/ "a1234b"))
 
 ;;-------------------------------------------------------------------------
 (test-section "uncapturing group")
@@ -529,6 +529,58 @@
 (test* "^(?i:a*).$" "Ab"
        (rxmatch-substring (#/^(?i:a*).$/ "Ab")))
 
+;;-------------------------------------------------------------------------
+(test-section "look-ahead assertion")
+
+(test* "/^(?=ab(de))(abd)(e)/" '("abde" "de" "abd" "e")
+       (match&list #/^(?=ab(de))(abd)(e)/ "abde"))
+(test* "/^(?!(ab)de|x)(abd)(f)/" '("abdf" #f "abd" "f")
+       (match&list #/^(?!(ab)de|x)(abd)(f)/ "abdf"))
+(test* "/^(?=(ab(cd)))(ab)/" '("ab" "abcd" "cd" "ab")
+       (match&list #/^(?=(ab(cd)))(ab)/ "abcd"))
+(test* "/\w+(?=\t)/" '("brown")
+       (match&list #/\w+(?=\t)/ "the quick brown\t fox"))
+(test* "/foo(?!bar)(.*)/" '("foolish see?" "lish see?")
+       (match&list #/foo(?!bar)(.*)/ "foobar is foolish see?"))
+(test* "/(?:(?!foo)...|^.{0,2})bar(.*)/" '("rowbar etc" " etc")
+       (match&list #/(?:(?!foo)...|^.{0,2})bar(.*)/ "foobar crowbar etc"))
+(test* "/(?:(?!foo)...|^.{0,2})bar(.*)/" '("barrel" "rel")
+       (match&list #/(?:(?!foo)...|^.{0,2})bar(.*)/ "barrel"))
+(test* "/(?:(?!foo)...|^.{0,2})bar(.*)/" '("2barrel" "rel")
+       (match&list #/(?:(?!foo)...|^.{0,2})bar(.*)/ "2barrel"))
+(test* "/(?:(?!foo)...|^.{0,2})bar(.*)/" '("A barrel" "rel")
+       (match&list #/(?:(?!foo)...|^.{0,2})bar(.*)/ "A barrel"))
+(test* "/^(\D*)(?=\d)(?!123)/" '("abc" "abc")
+       (match&list #/^(\D*)(?=\d)(?!123)/ "abc456"))
+(test* "/^(\D*)(?=\d)(?!123)/" #f
+       (match&list #/^(\D*)(?=\d)(?!123)/ "abc123"))
+(test* "/(?!^)abc/" '("abc")
+       (match&list #/(?!^)abc/ "the abc"))
+(test* "/(?!^)abc/" #f
+       (match&list #/(?!^)abc/ "abc"))
+(test* "/(?=^)abc/" '("abc")
+       (match&list #/(?=^)abc/ "abc"))
+(test* "/(?=^)abc/" #f
+       (match&list #/(?=^)abc/ "the abc"))
+(test* "/(\.\d\d((?=0)|\d(?=\d)))/" '(".23" ".23" "")
+       (match&list #/(\.\d\d((?=0)|\d(?=\d)))/ "1.230003938"))
+(test* "/(\.\d\d((?=0)|\d(?=\d)))/" '(".875" ".875" "5")
+       (match&list #/(\.\d\d((?=0)|\d(?=\d)))/ "1.875000282"))
+(test* "/(\.\d\d((?=0)|\d(?=\d)))/" #f
+       (match&list #/(\.\d\d((?=0)|\d(?=\d)))/ "1.235"))
+(test* "/^\D*(?!123)/" '("AB")
+       (match&list #/^\D*(?!123)/ "ABC123"))
+(test* "/^(\D*)(?=\d)(?!123)/" '("ABC" "ABC")
+       (match&list #/^(\D*)(?=\d)(?!123)/ "ABC445"))
+(test* "/^(\D*)(?=\d)(?!123)/" #f
+       (match&list #/^(\D*)(?=\d)(?!123)/ "ABC123"))
+(test* "/a(?!b)./" '("ad")
+       (match&list #/a(?!b)./ "abad"))
+(test* "/a(?=d)./" '("ad")
+       (match&list #/a(?=d)./ "abad"))
+(test* "/a(?=c|d)./" '("ad")
+       (match&list #/a(?=c|d)./ "abad"))
+
 ;;-------------------------------------------------------------------------
 (test-section "regexp macros")
 
