@@ -2,7 +2,7 @@
 ;; Test for SRFIs
 ;;
 
-;; $Id: srfi.scm,v 1.39 2004/11/01 21:51:05 shirok Exp $
+;; $Id: srfi.scm,v 1.41 2005/05/23 21:56:58 shirok Exp $
 
 (use gauche.test)
 
@@ -736,9 +736,9 @@
        (string-prefix-length "cancaNCAM" "cancancan"))
 (test* "string-prefix-length-ci" 8
        (string-prefix-length-ci "cancaNCAM" "cancancan"))
-(test* "string-suffix-length" 4
+(test* "string-suffix-length" 2
        (string-suffix-length "CanCan" "cankancan"))
-(test* "string-suffix-length-ci" 1
+(test* "string-suffix-length-ci" 5
        (string-suffix-length-ci "CanCan" "cankancan"))
 
 (test* "string-prefix?" #t    (string-prefix? "abcd" "abcdefg"))
@@ -1825,4 +1825,449 @@
 (test* "srfi-37 (operand)" '((i b) ("-" "foo" "bar"))
        (test-options "-i" "-" "foo" "-b" "bar"))
 
+;;-----------------------------------------------------------------------
+(test-section "srfi-42")
+
+(use srfi-42)
+
+;; tests took from examples of srfi-42 reference implementation.
+
+(test* "do-ec" 1
+       (let ((x 0)) (do-ec (set! x (+ x 1))) x))
+(test* "do-ec" 10
+       (let ((x 0)) (do-ec (:range i 10) (set! x (+ x 1))) x))
+(test* "do-ec" 45
+       (let ((x 0)) (do-ec (:range n 10) (:range k n) (set! x (+ x 1))) x))
+
+
+(test* "list-ec" '(1)
+       (list-ec 1))
+(test* "list-ec" '(0 1 2 3)
+       (list-ec (:range i 4) i))
+(test* "list-ec" '((0 0) (1 0) (1 1) (2 0) (2 1) (2 2))
+       (list-ec (:range n 3) (:range k (+ n 1)) (list n k)))
+(test* "list-ec" '((0 0) (2 0) (2 1) (2 2) (4 0) (4 1) (4 2) (4 3) (4 4))
+       (list-ec (:range n 5) (if (even? n)) (:range k (+ n 1)) (list n k)))
+(test* "list-ec" '((1 0) (1 1) (3 0) (3 1) (3 2) (3 3))
+       (list-ec (:range n 5) (not (even? n)) (:range k (+ n 1)) (list n k)))
+(test* "list-ec" '((4 0) (4 1) (4 2) (4 3) (4 4))
+       (list-ec (:range n 5) 
+                (and (even? n) (> n 2)) 
+                (:range k (+ n 1)) 
+                (list n k)))
+(test* "list-ec" '((0 0) (2 0) (2 1) (2 2) (4 0) (4 1) (4 2) (4 3) (4 4))
+       (list-ec (:range n 5) 
+                (or (even? n) (> n 3)) 
+                (:range k (+ n 1)) 
+                (list n k)))
+(test* "list-ec" 10
+       (let ((x 0)) (list-ec (:range n 10) (begin (set! x (+ x 1))) n) x))
+(test* "list-ec" '(0 0 1)
+       (list-ec (nested (:range n 3) (:range k n)) k))
+
+(test* "append-ec" '(a b)     (append-ec '(a b)))
+(test* "append-ec" '()        (append-ec (:range i 0) '(a b)))
+(test* "append-ec" '(a b)     (append-ec (:range i 1) '(a b)))
+(test* "append-ec" '(a b a b) (append-ec (:range i 2) '(a b)))
+
+(test* "string-ec" "a"  (string-ec #\a))
+(test* "string-ec" ""   (string-ec (:range i 0) #\a))
+(test* "string-ec" "a"  (string-ec (:range i 1) #\a))
+(test* "string-ec" "aa" (string-ec (:range i 2) #\a))
+
+(test* "string-append-ec" "ab"   (string-append-ec "ab"))
+(test* "string-append-ec" ""     (string-append-ec (:range i 0) "ab"))
+(test* "string-append-ec" "ab"   (string-append-ec (:range i 1) "ab"))
+(test* "string-append-ec" "abab" (string-append-ec (:range i 2) "ab"))
+
+(test* "vector-ec" '#(1)   (vector-ec 1))
+(test* "vector-ec" '#()    (vector-ec (:range i 0) i))
+(test* "vector-ec" '#(0)   (vector-ec (:range i 1) i))
+(test* "vector-ec" '#(0 1) (vector-ec (:range i 2) i))
+
+(test* "vector-of-length-ec" '#(1)   (vector-of-length-ec 1 1))
+(test* "vector-of-length-ec" '#()    (vector-of-length-ec 0 (:range i 0) i))
+(test* "vector-of-length-ec" '#(0)   (vector-of-length-ec 1 (:range i 1) i))
+(test* "vector-of-length-ec" '#(0 1) (vector-of-length-ec 2 (:range i 2) i))
+
+(test* "sum-ec" 1 (sum-ec 1))
+(test* "sum-ec" 0 (sum-ec (:range i 0) i))
+(test* "sum-ec" 0 (sum-ec (:range i 1) i))
+(test* "sum-ec" 1 (sum-ec (:range i 2) i))
+(test* "sum-ec" 3 (sum-ec (:range i 3) i))
+
+(test* "product-ec" 1 (product-ec 1))
+(test* "product-ec" 1 (product-ec (:range i 1 0) i))
+(test* "product-ec" 1 (product-ec (:range i 1 1) i))
+(test* "product-ec" 1 (product-ec (:range i 1 2) i))
+(test* "product-ec" 2 (product-ec (:range i 1 3) i))
+(test* "product-ec" 6 (product-ec (:range i 1 4) i))
+
+(test* "min-ec" 1 (min-ec 1))
+(test* "min-ec" 0 (min-ec (:range i 1) i))
+(test* "min-ec" 0 (min-ec (:range i 2) i))
+
+(test* "max-ec" 1 (max-ec 1))
+(test* "max-ec" 0 (max-ec (:range i 1) i))
+(test* "max-ec" 1 (max-ec (:range i 2) i))
+
+(test* "first-ec" 1  (first-ec #f 1))
+(test* "first-ec" #f (first-ec #f (:range i 0) i))
+(test* "first-ec" 0  (first-ec #f (:range i 1) i))
+(test* "first-ec" 0  (first-ec #f (:range i 2) i))
+(test* "first-ec" 0
+       (let ((last-i -1))
+         (first-ec #f (:range i 10) (begin (set! last-i i)) i)
+         last-i))
+
+(test* "last-ec" 1  (last-ec #f 1))
+(test* "last-ec" #f (last-ec #f (:range i 0) i))
+(test* "last-ec" 0  (last-ec #f (:range i 1) i))
+(test* "last-ec" 1  (last-ec #f (:range i 2) i))
+
+(test* "any-ec" #f (any?-ec #f))
+(test* "any-ec" #t (any?-ec #t))
+(test* "any-ec" #f (any?-ec (:range i 2 2) (even? i)))
+(test* "any-ec" #t (any?-ec (:range i 2 3) (even? i)))
+
+(test* "every-ec" #f (every?-ec #f))
+(test* "every-ec" #t (every?-ec #t))
+(test* "every-ec" #t (every?-ec (:range i 2 2) (even? i)))
+(test* "every-ec" #t (every?-ec (:range i 2 3) (even? i)))
+(test* "every-ec" #f (every?-ec (:range i 2 4) (even? i)))
+
+(test* "fold-ec" 285
+       (let ((sum-sqr (lambda (x result) (+ result (* x x)))))
+         (fold-ec 0 (:range i 10) i sum-sqr)))
+(test* "fold3-ec" 284
+       (let ((minus-1 (lambda (x) (- x 1)))
+             (sum-sqr (lambda (x result) (+ result (* x x)))))
+         (fold3-ec (error "wrong") (:range i 10) i minus-1 sum-sqr)))
+(test* "fold3-ec" 'infinity
+       (fold3-ec 'infinity (:range i 0) i min min))
+
+(test* ":list" '()
+       (list-ec (:list x '()) x))
+(test* ":list" '(1)
+       (list-ec (:list x '(1)) x))
+(test* ":list" '(1 2 3)
+       (list-ec (:list x '(1 2 3)) x))
+(test* ":list" '(1 2)
+       (list-ec (:list x '(1) '(2)) x))
+(test* ":list" '(1 2 3)
+       (list-ec (:list x '(1) '(2) '(3)) x))
+
+(test* ":string" '()
+       (list-ec (:string c "") c))
+(test* ":string" '(#\1)
+       (list-ec (:string c "1") c))
+(test* ":string" '(#\1 #\2 #\3)
+       (list-ec (:string c "123") c))
+(test* ":string" '(#\1 #\2)
+       (list-ec (:string c "1" "2") c))
+(test* ":string" '(#\1 #\2 #\3)
+       (list-ec (:string c "1" "2" "3") c))
+
+(test* ":vector" '()
+       (list-ec (:vector x (vector)) x))
+(test* ":vector" '(1)
+       (list-ec (:vector x (vector 1)) x))
+(test* ":vector" '(1 2 3)
+       (list-ec (:vector x (vector 1 2 3)) x))
+(test* ":vector" '(1 2)
+       (list-ec (:vector x (vector 1) (vector 2)) x))
+(test* ":vector" '(1 2 3)
+       (list-ec (:vector x (vector 1) (vector 2) (vector 3)) x))
+
+(test* ":range" '()
+       (list-ec (:range x -2) x))
+(test* ":range" '()
+       (list-ec (:range x -1) x))
+(test* ":range" '()
+       (list-ec (:range x  0) x))
+(test* ":range" '(0)
+       (list-ec (:range x  1) x))
+(test* ":range" '(0 1)
+       (list-ec (:range x  2) x))
+(test* ":range" '(0 1 2)
+       (list-ec (:range x  0  3) x))
+(test* ":range" '(1 2)
+       (list-ec (:range x  1  3) x))
+(test* ":range" '(-2)
+       (list-ec (:range x -2 -1) x))
+(test* ":range" '()
+       (list-ec (:range x -2 -2) x))
+(test* ":range" '(1 3)
+       (list-ec (:range x 1 5  2) x))
+(test* ":range" '(1 3 5)
+       (list-ec (:range x 1 6  2) x))
+(test* ":range" '(5 3)
+       (list-ec (:range x 5 1 -2) x))
+(test* ":range" '(6 4 2)
+       (list-ec (:range x 6 1 -2) x))
+
+(test* ":real-range" '(0. 1. 2.)
+       (list-ec (:real-range x 0.0 3.0)     x))
+(test* ":real-range" '(0. 1. 2.)
+       (list-ec (:real-range x 0   3.0)     x))
+(test* ":real-range" '(0. 1. 2.)
+       (list-ec (:real-range x 0   3   1.0) x))
+
+(test* ":char-range" "abcdefghijklmnopqrstuvwxyz"
+       (string-ec (:char-range c #\a #\z) c) )
+
+(sys-system "rm -f tmp1.o")
+
+(test* ":port" (list-ec (:range n 10) n)
+       (begin
+         (with-output-to-file "tmp1.o"
+           (lambda ()
+             (do-ec (:range n 10) (begin (write n) (newline)))))
+         (call-with-input-file "tmp1.o"
+           (lambda (port) (list-ec (:port x port read) x)))))
+
+(sys-system "rm -f tmp1.o")
+
+(test* ":do" '(0 1 2 3) (list-ec (:do ((i 0)) (< i 4) ((+ i 1))) i))
+(test* ":do" '(10 9 8 7)
+       (list-ec 
+        (:do (let ((x 'x)))
+             ((i 0)) 
+             (< i 4) 
+             (let ((j (- 10 i))))
+             #t
+             ((+ i 1)))
+        j))
+
+(test* ":let" '(1) (list-ec (:let x 1) x))
+(test* ":let" '(2) (list-ec (:let x 1) (:let y (+ x 1)) y))
+(test* ":let" '(2) (list-ec (:let x 1) (:let x (+ x 1)) x))
+
+(test* ":parallel" '((1 a) (2 b) (3 c))
+       (list-ec (:parallel (:range i 1 10) (:list x '(a b c))) (list i x)))
+(test* ":until" '(1 2 3 4 5)
+       (list-ec (:until (:range i 1 10) (>= i 5)) i))
+(test* ":while" '(1 2 3 4)
+       (list-ec (:while (:list i '(1 2 3 4 5 6 7 8 9)) (< i 5)) i))
+(test* ":until" '(1 2 3 4 5)
+       (list-ec (:until (:list i '(1 2 3 4 5 6 7 8 9)) (>= i 5)) i))
+
+(test* ":while and :parallel" '((1 1) (2 2) (3 3) (4 4))
+       (list-ec (:while (:parallel (:range i 1 10)
+                                   (:list j '(1 2 3 4 5 6 7 8 9)))
+                        (< i 5))
+                (list i j)))
+(test* ":until and :parallel" '((1 1) (2 2) (3 3) (4 4) (5 5))
+       (list-ec (:until (:parallel (:range i 1 10)
+                                   (:list j '(1 2 3 4 5 6 7 8 9)))
+                        (>= i 5))
+                (list i j)))
+(test* ":while stopping loop" 5
+       (let ((n 0))
+         (do-ec (:while (:range i 1 10) (begin (set! n (+ n 1)) (< i 5)))
+                (if #f #f))
+         n))
+(test* ":until stopping loop" 5
+       (let ((n 0))
+         (do-ec (:until (:range i 1 10) (begin (set! n (+ n 1)) (>= i 5)))
+                (if #f #f))
+         n))
+(test* ":while stopping loop" 5
+       (let ((n 0))
+         (do-ec (:while (:parallel (:range i 1 10)
+                                   (:do () (begin (set! n (+ n 1)) #t) ()))
+                        (< i 5))
+                (if #f #f))
+         n))
+(test* ":until stopping loop" 5
+       (let ((n 0))
+         (do-ec (:until (:parallel (:range i 1 10)
+                                   (:do () (begin (set! n (+ n 1)) #t) ()))
+                        (>= i 5))
+                (if #f #f))
+         n))
+
+(test* ": list" '(a b)     (list-ec (: c '(a b)) c))
+(test* ": list" '(a b c d) (list-ec (: c '(a b) '(c d)) c))
+
+(test* ": string" '(#\a #\b)         (list-ec (: c "ab") c))
+(test* ": string" '(#\a #\b #\c #\d) (list-ec (: c "ab" "cd") c))
+
+(test* ": vector" '(a b)   (list-ec (: c (vector 'a 'b)) c))
+(test* ": vector" '(a b c) (list-ec (: c (vector 'a 'b) (vector 'c)) c))
+
+(test* ": range" '()  (list-ec (: i 0) i))
+(test* ": range" '(0) (list-ec (: i 1) i))
+(test* ": range" '(0 1 2 3 4 5 6 7 8 9) (list-ec (: i 10) i))
+(test* ": range" '(1) (list-ec (: i 1 2) i))
+(test* ": range" '(1) (list-ec (: i 1 2 3) i))
+(test* ": range" '(1 4 7) (list-ec (: i 1 9 3) i))
+
+(test* ": real-range" '(0. 0.2 0.4 0.6 0.8) (list-ec (: i 0.0 1.0 0.2) i)
+       (lambda (x y)
+         (every (lambda (p q) (< (abs (- p q)) 1.0e-5)) x y)))
+(test* ": char" '(#\a #\b #\c) (list-ec (: c #\a #\c) c))
+
+(sys-system "rm -f tmp1.o")
+(test* ": port" (list-ec (:range n 10) n)
+       (begin
+         (with-output-to-file "tmp1.o"
+           (lambda ()
+             (do-ec (:range n 10) (begin (write n) (newline)))))
+         (call-with-input-file "tmp1.o"
+           (lambda (port) (list-ec (: x port read) x)))))
+             
+(sys-system "rm -f tmp1.o")
+(test* ": port" (list-ec (:range n 10) n)
+       (begin
+         (with-output-to-file "tmp1.o"
+           (lambda ()
+             (do-ec (:range n 10) (begin (write n) (newline)))))
+         (call-with-input-file "tmp1.o"
+           (lambda (port) (list-ec (: x port) x)))))       
+
+(sys-system "rm -f tmp1.o")
+
+(test* ":list index" '((a 0) (b 1))
+       (list-ec (:list c (index i) '(a b)) (list c i)))
+(test* ":string index" '((#\a 0))
+       (list-ec (:string c (index i) "a") (list c i)))
+(test* ":vector index" '((a 0))
+       (list-ec (:vector c (index i) (vector 'a)) (list c i)))
+(test* ":range index" '((0 0) (-1 1) (-2 2))
+       (list-ec (:range i (index j) 0 -3 -1) (list i j)) )
+(test* ":real-range index" '((0. 0) (0.2 1) (0.4 2) (0.6 3) (0.8 4))
+       (list-ec (:real-range i (index j) 0 1 0.2) (list i j))
+       (lambda (x y)
+         (every (lambda (p q) (and (< (abs (- (car p) (car q))) 1e-5)
+                                   (= (cadr p) (cadr q))))
+                x y)))
+(test* ":char-range index" '((#\a 0) (#\b 1) (#\c 2))
+       (list-ec (:char-range c (index i) #\a #\c) (list c i)) )
+(test* ": index" '((a 0) (b 1) (c 2) (d 3))
+       (list-ec (: x (index i) '(a b c d)) (list x i)) )
+(sys-system "rm -f tmp1.o")
+(test* ": index port"
+       '((0 0) (1 1) (2 2) (3 3) (4 4) (5 5) (6 6) (7 7) (8 8) (9 9))
+       (begin
+         (with-output-to-file "tmp1.o"
+           (lambda ()
+             (do-ec (:range n 10) (begin (write n) (newline)))))
+         (call-with-input-file "tmp1.o"
+           (lambda (port) (list-ec (: x (index i) port) (list x i))))))
+(sys-system "rm -f tmp1.o")
+
+(test* "example 1" '(0 1 4 9 16)
+       (list-ec (: i 5) (* i i)))
+(test* "example 2" '((1 0) (2 0) (2 1) (3 0) (3 1) (3 2))
+       (list-ec (: n 1 4) (: i n) (list n i)))
+(test* "example 3" '((#\a 0) (#\b 1) (#\c 2))
+       (list-ec (: x (index i) "abc") (list x i)) )
+(test* "example 4" '((#\a . 0) (#\b . 1))
+       (list-ec (:string c (index i) "a" "b") (cons c i)) )
+
+(test* ":range :range" '(0 0 1 0 1 2 0 1 2 3)
+       (list-ec (:range x 5) (:range x x) x))
+(test* ":list :" '(0 1 #\2 #\3 4)
+       (list-ec (:list x '(2 "23" (4))) (: y x) y))
+(test* ":parallel :integers :do" '((0 10) (1 9) (2 8) (3 7) (4 6))
+       (list-ec (:parallel (:integers x) 
+                           (:do ((i 10)) (< x i) ((- i 1))))
+                (list x i)))
+
+(let ()
+  (define (factorial n) ; n * (n-1) * .. * 1 for n >= 0
+    (product-ec (:range k 2 (+ n 1)) k) )
+
+  (test* "factorial" 1   (factorial 0))
+  (test* "factorial" 1   (factorial 1))
+  (test* "factorial" 6   (factorial 3))
+  (test* "factorial" 120 (factorial 5))
+  )
+
+(use srfi-4)
+(let ()
+  (define (eratosthenes n) ; primes in {2..n-1} for n >= 1
+    (let ((p? (make-u8vector n 1)))
+      (do-ec (:range k 2 n)
+             (if (= (u8vector-ref p? k) 1))
+             (:range i (* 2 k) n k)
+             (u8vector-set! p? i 0))
+      (list-ec (:range k 2 n) (if (= (u8vector-ref p? k) 1)) k) ))
+  (test* "eratosthenes" '(2 3 5 7 11 13 17 19 23 29 31 37 41 43 47)
+         (eratosthenes 50))
+  (test* "eratosthenes" 9592 (length (eratosthenes 100000)))
+  )
+
+(let ()
+  (define (pythagoras n) ; a, b, c s.t. 1 <= a <= b <= c <= n, a^2 + b^2 = c^2
+    (list-ec 
+     (:let sqr-n (* n n))
+     (:range a 1 (+ n 1))
+     (:let sqr-a (* a a))
+     (:range b a (+ n 1)) 
+     (:let sqr-c (+ sqr-a (* b b)))
+     (if (<= sqr-c sqr-n))
+     (:range c b (+ n 1))
+     (if (= (* c c) sqr-c))
+     (list a b c) ))
+  (test* "pythagoras" '((3 4 5) (5 12 13) (6 8 10) (9 12 15))
+         (pythagoras 15))
+  (test* "pythagoras" 127 (length (pythagoras 200)))
+  )
+
+(let ()
+  (define (qsort xs) ; stable
+    (if (null? xs)
+      '()
+      (let ((pivot (car xs)) (xrest (cdr xs)))
+        (append
+         (qsort (list-ec (:list x xrest) (if (<  x pivot)) x))
+         (list pivot)
+         (qsort (list-ec (:list x xrest) (if (>= x pivot)) x)) ))))
+  (test* "qsort" '(1 1 2 2 3 3 4 4 5 5) (qsort '(1 5 4 2 4 5 3 2 1 3)))
+  )
+
+(let ()
+  (define (pi-BBP m) ; approx. of pi within 16^-m (Bailey-Borwein-Plouffe)
+    (sum-ec 
+     (:range n 0 (+ m 1))
+     (:let n8 (* 8 n))
+     (* (- (/ 4 (+ n8 1))
+           (+ (/ 2 (+ n8 4))
+              (/ 1 (+ n8 5))
+              (/ 1 (+ n8 6))))
+        (/ 1 (expt 16 n)) )))
+  ;; NB: Gauche doesn't have exact rational (yet), so we have to torelate
+  ;; rounding error
+  (test* "pi-BBP" (/ 40413742330349316707 12864093722915635200)
+         (pi-BBP 5)
+         (lambda (x y) (< (abs (- x y)) 1e-10)))
+  )
+
+(let ()
+  (define (read-line port) ; next line (incl. #\newline) of port
+    (let ((line
+           (string-ec 
+            (:until (:port c port read-char)
+                    (char=? c #\newline) )
+            c )))
+      (if (string=? line "")
+        (read-char port) ; eof-object
+        line )))
+
+  (define (read-lines filename) ; list of all lines
+    (call-with-input-file filename
+      (lambda (port)
+        (list-ec (:port line port read-line) line) )))
+
+  (sys-system "rm -f tmp1.o")
+  (test* "read-lines" (list-ec (:char-range c #\0 #\9) (string c #\newline))
+         (begin
+           (with-output-to-file "tmp1.o"
+             (lambda ()
+               (do-ec (:range n 10) (begin (write n) (newline)))))
+           (read-lines "tmp1.o")))
+  )
+
 (test-end)
